1 //=== RecordLayoutBuilder.cpp - Helper class for building record layouts ---==//
2 //
3 //                     The LLVM Compiler Infrastructure
4 //
5 // This file is distributed under the University of Illinois Open Source
6 // License. See LICENSE.TXT for details.
7 //
8 //===----------------------------------------------------------------------===//
9 
10 #include "clang/AST/RecordLayout.h"
11 #include "clang/AST/ASTContext.h"
12 #include "clang/AST/Attr.h"
13 #include "clang/AST/CXXInheritance.h"
14 #include "clang/AST/Decl.h"
15 #include "clang/AST/DeclCXX.h"
16 #include "clang/AST/DeclObjC.h"
17 #include "clang/AST/Expr.h"
18 #include "clang/Basic/TargetInfo.h"
19 #include "clang/Sema/SemaDiagnostic.h"
20 #include "llvm/ADT/SmallSet.h"
21 #include "llvm/Support/Format.h"
22 #include "llvm/Support/MathExtras.h"
23 
24 using namespace clang;
25 
26 namespace {
27 
28 /// BaseSubobjectInfo - Represents a single base subobject in a complete class.
29 /// For a class hierarchy like
30 ///
31 /// class A { };
32 /// class B : A { };
33 /// class C : A, B { };
34 ///
35 /// The BaseSubobjectInfo graph for C will have three BaseSubobjectInfo
36 /// instances, one for B and two for A.
37 ///
38 /// If a base is virtual, it will only have one BaseSubobjectInfo allocated.
39 struct BaseSubobjectInfo {
40   /// Class - The class for this base info.
41   const CXXRecordDecl *Class;
42 
43   /// IsVirtual - Whether the BaseInfo represents a virtual base or not.
44   bool IsVirtual;
45 
46   /// Bases - Information about the base subobjects.
47   SmallVector<BaseSubobjectInfo*, 4> Bases;
48 
49   /// PrimaryVirtualBaseInfo - Holds the base info for the primary virtual base
50   /// of this base info (if one exists).
51   BaseSubobjectInfo *PrimaryVirtualBaseInfo;
52 
53   // FIXME: Document.
54   const BaseSubobjectInfo *Derived;
55 };
56 
57 /// \brief Externally provided layout. Typically used when the AST source, such
58 /// as DWARF, lacks all the information that was available at compile time, such
59 /// as alignment attributes on fields and pragmas in effect.
60 struct ExternalLayout {
61   ExternalLayout() : Size(0), Align(0) {}
62 
63   /// \brief Overall record size in bits.
64   uint64_t Size;
65 
66   /// \brief Overall record alignment in bits.
67   uint64_t Align;
68 
69   /// \brief Record field offsets in bits.
70   llvm::DenseMap<const FieldDecl *, uint64_t> FieldOffsets;
71 
72   /// \brief Direct, non-virtual base offsets.
73   llvm::DenseMap<const CXXRecordDecl *, CharUnits> BaseOffsets;
74 
75   /// \brief Virtual base offsets.
76   llvm::DenseMap<const CXXRecordDecl *, CharUnits> VirtualBaseOffsets;
77 
78   /// Get the offset of the given field. The external source must provide
79   /// entries for all fields in the record.
80   uint64_t getExternalFieldOffset(const FieldDecl *FD) {
81     assert(FieldOffsets.count(FD) &&
82            "Field does not have an external offset");
83     return FieldOffsets[FD];
84   }
85 
86   bool getExternalNVBaseOffset(const CXXRecordDecl *RD, CharUnits &BaseOffset) {
87     auto Known = BaseOffsets.find(RD);
88     if (Known == BaseOffsets.end())
89       return false;
90     BaseOffset = Known->second;
91     return true;
92   }
93 
94   bool getExternalVBaseOffset(const CXXRecordDecl *RD, CharUnits &BaseOffset) {
95     auto Known = VirtualBaseOffsets.find(RD);
96     if (Known == VirtualBaseOffsets.end())
97       return false;
98     BaseOffset = Known->second;
99     return true;
100   }
101 };
102 
103 /// EmptySubobjectMap - Keeps track of which empty subobjects exist at different
104 /// offsets while laying out a C++ class.
105 class EmptySubobjectMap {
106   const ASTContext &Context;
107   uint64_t CharWidth;
108 
109   /// Class - The class whose empty entries we're keeping track of.
110   const CXXRecordDecl *Class;
111 
112   /// EmptyClassOffsets - A map from offsets to empty record decls.
113   typedef llvm::TinyPtrVector<const CXXRecordDecl *> ClassVectorTy;
114   typedef llvm::DenseMap<CharUnits, ClassVectorTy> EmptyClassOffsetsMapTy;
115   EmptyClassOffsetsMapTy EmptyClassOffsets;
116 
117   /// MaxEmptyClassOffset - The highest offset known to contain an empty
118   /// base subobject.
119   CharUnits MaxEmptyClassOffset;
120 
121   /// ComputeEmptySubobjectSizes - Compute the size of the largest base or
122   /// member subobject that is empty.
123   void ComputeEmptySubobjectSizes();
124 
125   void AddSubobjectAtOffset(const CXXRecordDecl *RD, CharUnits Offset);
126 
127   void UpdateEmptyBaseSubobjects(const BaseSubobjectInfo *Info,
128                                  CharUnits Offset, bool PlacingEmptyBase);
129 
130   void UpdateEmptyFieldSubobjects(const CXXRecordDecl *RD,
131                                   const CXXRecordDecl *Class,
132                                   CharUnits Offset);
133   void UpdateEmptyFieldSubobjects(const FieldDecl *FD, CharUnits Offset);
134 
135   /// AnyEmptySubobjectsBeyondOffset - Returns whether there are any empty
136   /// subobjects beyond the given offset.
137   bool AnyEmptySubobjectsBeyondOffset(CharUnits Offset) const {
138     return Offset <= MaxEmptyClassOffset;
139   }
140 
141   CharUnits
142   getFieldOffset(const ASTRecordLayout &Layout, unsigned FieldNo) const {
143     uint64_t FieldOffset = Layout.getFieldOffset(FieldNo);
144     assert(FieldOffset % CharWidth == 0 &&
145            "Field offset not at char boundary!");
146 
147     return Context.toCharUnitsFromBits(FieldOffset);
148   }
149 
150 protected:
151   bool CanPlaceSubobjectAtOffset(const CXXRecordDecl *RD,
152                                  CharUnits Offset) const;
153 
154   bool CanPlaceBaseSubobjectAtOffset(const BaseSubobjectInfo *Info,
155                                      CharUnits Offset);
156 
157   bool CanPlaceFieldSubobjectAtOffset(const CXXRecordDecl *RD,
158                                       const CXXRecordDecl *Class,
159                                       CharUnits Offset) const;
160   bool CanPlaceFieldSubobjectAtOffset(const FieldDecl *FD,
161                                       CharUnits Offset) const;
162 
163 public:
164   /// This holds the size of the largest empty subobject (either a base
165   /// or a member). Will be zero if the record being built doesn't contain
166   /// any empty classes.
167   CharUnits SizeOfLargestEmptySubobject;
168 
169   EmptySubobjectMap(const ASTContext &Context, const CXXRecordDecl *Class)
170   : Context(Context), CharWidth(Context.getCharWidth()), Class(Class) {
171       ComputeEmptySubobjectSizes();
172   }
173 
174   /// CanPlaceBaseAtOffset - Return whether the given base class can be placed
175   /// at the given offset.
176   /// Returns false if placing the record will result in two components
177   /// (direct or indirect) of the same type having the same offset.
178   bool CanPlaceBaseAtOffset(const BaseSubobjectInfo *Info,
179                             CharUnits Offset);
180 
181   /// CanPlaceFieldAtOffset - Return whether a field can be placed at the given
182   /// offset.
183   bool CanPlaceFieldAtOffset(const FieldDecl *FD, CharUnits Offset);
184 };
185 
186 void EmptySubobjectMap::ComputeEmptySubobjectSizes() {
187   // Check the bases.
188   for (const CXXBaseSpecifier &Base : Class->bases()) {
189     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
190 
191     CharUnits EmptySize;
192     const ASTRecordLayout &Layout = Context.getASTRecordLayout(BaseDecl);
193     if (BaseDecl->isEmpty()) {
194       // If the class decl is empty, get its size.
195       EmptySize = Layout.getSize();
196     } else {
197       // Otherwise, we get the largest empty subobject for the decl.
198       EmptySize = Layout.getSizeOfLargestEmptySubobject();
199     }
200 
201     if (EmptySize > SizeOfLargestEmptySubobject)
202       SizeOfLargestEmptySubobject = EmptySize;
203   }
204 
205   // Check the fields.
206   for (const FieldDecl *FD : Class->fields()) {
207     const RecordType *RT =
208         Context.getBaseElementType(FD->getType())->getAs<RecordType>();
209 
210     // We only care about record types.
211     if (!RT)
212       continue;
213 
214     CharUnits EmptySize;
215     const CXXRecordDecl *MemberDecl = RT->getAsCXXRecordDecl();
216     const ASTRecordLayout &Layout = Context.getASTRecordLayout(MemberDecl);
217     if (MemberDecl->isEmpty()) {
218       // If the class decl is empty, get its size.
219       EmptySize = Layout.getSize();
220     } else {
221       // Otherwise, we get the largest empty subobject for the decl.
222       EmptySize = Layout.getSizeOfLargestEmptySubobject();
223     }
224 
225     if (EmptySize > SizeOfLargestEmptySubobject)
226       SizeOfLargestEmptySubobject = EmptySize;
227   }
228 }
229 
230 bool
231 EmptySubobjectMap::CanPlaceSubobjectAtOffset(const CXXRecordDecl *RD,
232                                              CharUnits Offset) const {
233   // We only need to check empty bases.
234   if (!RD->isEmpty())
235     return true;
236 
237   EmptyClassOffsetsMapTy::const_iterator I = EmptyClassOffsets.find(Offset);
238   if (I == EmptyClassOffsets.end())
239     return true;
240 
241   const ClassVectorTy &Classes = I->second;
242   if (std::find(Classes.begin(), Classes.end(), RD) == Classes.end())
243     return true;
244 
245   // There is already an empty class of the same type at this offset.
246   return false;
247 }
248 
249 void EmptySubobjectMap::AddSubobjectAtOffset(const CXXRecordDecl *RD,
250                                              CharUnits Offset) {
251   // We only care about empty bases.
252   if (!RD->isEmpty())
253     return;
254 
255   // If we have empty structures inside a union, we can assign both
256   // the same offset. Just avoid pushing them twice in the list.
257   ClassVectorTy &Classes = EmptyClassOffsets[Offset];
258   if (std::find(Classes.begin(), Classes.end(), RD) != Classes.end())
259     return;
260 
261   Classes.push_back(RD);
262 
263   // Update the empty class offset.
264   if (Offset > MaxEmptyClassOffset)
265     MaxEmptyClassOffset = Offset;
266 }
267 
268 bool
269 EmptySubobjectMap::CanPlaceBaseSubobjectAtOffset(const BaseSubobjectInfo *Info,
270                                                  CharUnits Offset) {
271   // We don't have to keep looking past the maximum offset that's known to
272   // contain an empty class.
273   if (!AnyEmptySubobjectsBeyondOffset(Offset))
274     return true;
275 
276   if (!CanPlaceSubobjectAtOffset(Info->Class, Offset))
277     return false;
278 
279   // Traverse all non-virtual bases.
280   const ASTRecordLayout &Layout = Context.getASTRecordLayout(Info->Class);
281   for (const BaseSubobjectInfo *Base : Info->Bases) {
282     if (Base->IsVirtual)
283       continue;
284 
285     CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base->Class);
286 
287     if (!CanPlaceBaseSubobjectAtOffset(Base, BaseOffset))
288       return false;
289   }
290 
291   if (Info->PrimaryVirtualBaseInfo) {
292     BaseSubobjectInfo *PrimaryVirtualBaseInfo = Info->PrimaryVirtualBaseInfo;
293 
294     if (Info == PrimaryVirtualBaseInfo->Derived) {
295       if (!CanPlaceBaseSubobjectAtOffset(PrimaryVirtualBaseInfo, Offset))
296         return false;
297     }
298   }
299 
300   // Traverse all member variables.
301   unsigned FieldNo = 0;
302   for (CXXRecordDecl::field_iterator I = Info->Class->field_begin(),
303        E = Info->Class->field_end(); I != E; ++I, ++FieldNo) {
304     if (I->isBitField())
305       continue;
306 
307     CharUnits FieldOffset = Offset + getFieldOffset(Layout, FieldNo);
308     if (!CanPlaceFieldSubobjectAtOffset(*I, FieldOffset))
309       return false;
310   }
311 
312   return true;
313 }
314 
315 void EmptySubobjectMap::UpdateEmptyBaseSubobjects(const BaseSubobjectInfo *Info,
316                                                   CharUnits Offset,
317                                                   bool PlacingEmptyBase) {
318   if (!PlacingEmptyBase && Offset >= SizeOfLargestEmptySubobject) {
319     // We know that the only empty subobjects that can conflict with empty
320     // subobject of non-empty bases, are empty bases that can be placed at
321     // offset zero. Because of this, we only need to keep track of empty base
322     // subobjects with offsets less than the size of the largest empty
323     // subobject for our class.
324     return;
325   }
326 
327   AddSubobjectAtOffset(Info->Class, Offset);
328 
329   // Traverse all non-virtual bases.
330   const ASTRecordLayout &Layout = Context.getASTRecordLayout(Info->Class);
331   for (const BaseSubobjectInfo *Base : Info->Bases) {
332     if (Base->IsVirtual)
333       continue;
334 
335     CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base->Class);
336     UpdateEmptyBaseSubobjects(Base, BaseOffset, PlacingEmptyBase);
337   }
338 
339   if (Info->PrimaryVirtualBaseInfo) {
340     BaseSubobjectInfo *PrimaryVirtualBaseInfo = Info->PrimaryVirtualBaseInfo;
341 
342     if (Info == PrimaryVirtualBaseInfo->Derived)
343       UpdateEmptyBaseSubobjects(PrimaryVirtualBaseInfo, Offset,
344                                 PlacingEmptyBase);
345   }
346 
347   // Traverse all member variables.
348   unsigned FieldNo = 0;
349   for (CXXRecordDecl::field_iterator I = Info->Class->field_begin(),
350        E = Info->Class->field_end(); I != E; ++I, ++FieldNo) {
351     if (I->isBitField())
352       continue;
353 
354     CharUnits FieldOffset = Offset + getFieldOffset(Layout, FieldNo);
355     UpdateEmptyFieldSubobjects(*I, FieldOffset);
356   }
357 }
358 
359 bool EmptySubobjectMap::CanPlaceBaseAtOffset(const BaseSubobjectInfo *Info,
360                                              CharUnits Offset) {
361   // If we know this class doesn't have any empty subobjects we don't need to
362   // bother checking.
363   if (SizeOfLargestEmptySubobject.isZero())
364     return true;
365 
366   if (!CanPlaceBaseSubobjectAtOffset(Info, Offset))
367     return false;
368 
369   // We are able to place the base at this offset. Make sure to update the
370   // empty base subobject map.
371   UpdateEmptyBaseSubobjects(Info, Offset, Info->Class->isEmpty());
372   return true;
373 }
374 
375 bool
376 EmptySubobjectMap::CanPlaceFieldSubobjectAtOffset(const CXXRecordDecl *RD,
377                                                   const CXXRecordDecl *Class,
378                                                   CharUnits Offset) const {
379   // We don't have to keep looking past the maximum offset that's known to
380   // contain an empty class.
381   if (!AnyEmptySubobjectsBeyondOffset(Offset))
382     return true;
383 
384   if (!CanPlaceSubobjectAtOffset(RD, Offset))
385     return false;
386 
387   const ASTRecordLayout &Layout = Context.getASTRecordLayout(RD);
388 
389   // Traverse all non-virtual bases.
390   for (const CXXBaseSpecifier &Base : RD->bases()) {
391     if (Base.isVirtual())
392       continue;
393 
394     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
395 
396     CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(BaseDecl);
397     if (!CanPlaceFieldSubobjectAtOffset(BaseDecl, Class, BaseOffset))
398       return false;
399   }
400 
401   if (RD == Class) {
402     // This is the most derived class, traverse virtual bases as well.
403     for (const CXXBaseSpecifier &Base : RD->vbases()) {
404       const CXXRecordDecl *VBaseDecl = Base.getType()->getAsCXXRecordDecl();
405 
406       CharUnits VBaseOffset = Offset + Layout.getVBaseClassOffset(VBaseDecl);
407       if (!CanPlaceFieldSubobjectAtOffset(VBaseDecl, Class, VBaseOffset))
408         return false;
409     }
410   }
411 
412   // Traverse all member variables.
413   unsigned FieldNo = 0;
414   for (CXXRecordDecl::field_iterator I = RD->field_begin(), E = RD->field_end();
415        I != E; ++I, ++FieldNo) {
416     if (I->isBitField())
417       continue;
418 
419     CharUnits FieldOffset = Offset + getFieldOffset(Layout, FieldNo);
420 
421     if (!CanPlaceFieldSubobjectAtOffset(*I, FieldOffset))
422       return false;
423   }
424 
425   return true;
426 }
427 
428 bool
429 EmptySubobjectMap::CanPlaceFieldSubobjectAtOffset(const FieldDecl *FD,
430                                                   CharUnits Offset) const {
431   // We don't have to keep looking past the maximum offset that's known to
432   // contain an empty class.
433   if (!AnyEmptySubobjectsBeyondOffset(Offset))
434     return true;
435 
436   QualType T = FD->getType();
437   if (const CXXRecordDecl *RD = T->getAsCXXRecordDecl())
438     return CanPlaceFieldSubobjectAtOffset(RD, RD, Offset);
439 
440   // If we have an array type we need to look at every element.
441   if (const ConstantArrayType *AT = Context.getAsConstantArrayType(T)) {
442     QualType ElemTy = Context.getBaseElementType(AT);
443     const RecordType *RT = ElemTy->getAs<RecordType>();
444     if (!RT)
445       return true;
446 
447     const CXXRecordDecl *RD = RT->getAsCXXRecordDecl();
448     const ASTRecordLayout &Layout = Context.getASTRecordLayout(RD);
449 
450     uint64_t NumElements = Context.getConstantArrayElementCount(AT);
451     CharUnits ElementOffset = Offset;
452     for (uint64_t I = 0; I != NumElements; ++I) {
453       // We don't have to keep looking past the maximum offset that's known to
454       // contain an empty class.
455       if (!AnyEmptySubobjectsBeyondOffset(ElementOffset))
456         return true;
457 
458       if (!CanPlaceFieldSubobjectAtOffset(RD, RD, ElementOffset))
459         return false;
460 
461       ElementOffset += Layout.getSize();
462     }
463   }
464 
465   return true;
466 }
467 
468 bool
469 EmptySubobjectMap::CanPlaceFieldAtOffset(const FieldDecl *FD,
470                                          CharUnits Offset) {
471   if (!CanPlaceFieldSubobjectAtOffset(FD, Offset))
472     return false;
473 
474   // We are able to place the member variable at this offset.
475   // Make sure to update the empty base subobject map.
476   UpdateEmptyFieldSubobjects(FD, Offset);
477   return true;
478 }
479 
480 void EmptySubobjectMap::UpdateEmptyFieldSubobjects(const CXXRecordDecl *RD,
481                                                    const CXXRecordDecl *Class,
482                                                    CharUnits Offset) {
483   // We know that the only empty subobjects that can conflict with empty
484   // field subobjects are subobjects of empty bases that can be placed at offset
485   // zero. Because of this, we only need to keep track of empty field
486   // subobjects with offsets less than the size of the largest empty
487   // subobject for our class.
488   if (Offset >= SizeOfLargestEmptySubobject)
489     return;
490 
491   AddSubobjectAtOffset(RD, Offset);
492 
493   const ASTRecordLayout &Layout = Context.getASTRecordLayout(RD);
494 
495   // Traverse all non-virtual bases.
496   for (const CXXBaseSpecifier &Base : RD->bases()) {
497     if (Base.isVirtual())
498       continue;
499 
500     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
501 
502     CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(BaseDecl);
503     UpdateEmptyFieldSubobjects(BaseDecl, Class, BaseOffset);
504   }
505 
506   if (RD == Class) {
507     // This is the most derived class, traverse virtual bases as well.
508     for (const CXXBaseSpecifier &Base : RD->vbases()) {
509       const CXXRecordDecl *VBaseDecl = Base.getType()->getAsCXXRecordDecl();
510 
511       CharUnits VBaseOffset = Offset + Layout.getVBaseClassOffset(VBaseDecl);
512       UpdateEmptyFieldSubobjects(VBaseDecl, Class, VBaseOffset);
513     }
514   }
515 
516   // Traverse all member variables.
517   unsigned FieldNo = 0;
518   for (CXXRecordDecl::field_iterator I = RD->field_begin(), E = RD->field_end();
519        I != E; ++I, ++FieldNo) {
520     if (I->isBitField())
521       continue;
522 
523     CharUnits FieldOffset = Offset + getFieldOffset(Layout, FieldNo);
524 
525     UpdateEmptyFieldSubobjects(*I, FieldOffset);
526   }
527 }
528 
529 void EmptySubobjectMap::UpdateEmptyFieldSubobjects(const FieldDecl *FD,
530                                                    CharUnits Offset) {
531   QualType T = FD->getType();
532   if (const CXXRecordDecl *RD = T->getAsCXXRecordDecl()) {
533     UpdateEmptyFieldSubobjects(RD, RD, Offset);
534     return;
535   }
536 
537   // If we have an array type we need to update every element.
538   if (const ConstantArrayType *AT = Context.getAsConstantArrayType(T)) {
539     QualType ElemTy = Context.getBaseElementType(AT);
540     const RecordType *RT = ElemTy->getAs<RecordType>();
541     if (!RT)
542       return;
543 
544     const CXXRecordDecl *RD = RT->getAsCXXRecordDecl();
545     const ASTRecordLayout &Layout = Context.getASTRecordLayout(RD);
546 
547     uint64_t NumElements = Context.getConstantArrayElementCount(AT);
548     CharUnits ElementOffset = Offset;
549 
550     for (uint64_t I = 0; I != NumElements; ++I) {
551       // We know that the only empty subobjects that can conflict with empty
552       // field subobjects are subobjects of empty bases that can be placed at
553       // offset zero. Because of this, we only need to keep track of empty field
554       // subobjects with offsets less than the size of the largest empty
555       // subobject for our class.
556       if (ElementOffset >= SizeOfLargestEmptySubobject)
557         return;
558 
559       UpdateEmptyFieldSubobjects(RD, RD, ElementOffset);
560       ElementOffset += Layout.getSize();
561     }
562   }
563 }
564 
565 typedef llvm::SmallPtrSet<const CXXRecordDecl*, 4> ClassSetTy;
566 
567 class ItaniumRecordLayoutBuilder {
568 protected:
569   // FIXME: Remove this and make the appropriate fields public.
570   friend class clang::ASTContext;
571 
572   const ASTContext &Context;
573 
574   EmptySubobjectMap *EmptySubobjects;
575 
576   /// Size - The current size of the record layout.
577   uint64_t Size;
578 
579   /// Alignment - The current alignment of the record layout.
580   CharUnits Alignment;
581 
582   /// \brief The alignment if attribute packed is not used.
583   CharUnits UnpackedAlignment;
584 
585   SmallVector<uint64_t, 16> FieldOffsets;
586 
587   /// \brief Whether the external AST source has provided a layout for this
588   /// record.
589   unsigned UseExternalLayout : 1;
590 
591   /// \brief Whether we need to infer alignment, even when we have an
592   /// externally-provided layout.
593   unsigned InferAlignment : 1;
594 
595   /// Packed - Whether the record is packed or not.
596   unsigned Packed : 1;
597 
598   unsigned IsUnion : 1;
599 
600   unsigned IsMac68kAlign : 1;
601 
602   unsigned IsMsStruct : 1;
603 
604   /// UnfilledBitsInLastUnit - If the last field laid out was a bitfield,
605   /// this contains the number of bits in the last unit that can be used for
606   /// an adjacent bitfield if necessary.  The unit in question is usually
607   /// a byte, but larger units are used if IsMsStruct.
608   unsigned char UnfilledBitsInLastUnit;
609   /// LastBitfieldTypeSize - If IsMsStruct, represents the size of the type
610   /// of the previous field if it was a bitfield.
611   unsigned char LastBitfieldTypeSize;
612 
613   /// MaxFieldAlignment - The maximum allowed field alignment. This is set by
614   /// #pragma pack.
615   CharUnits MaxFieldAlignment;
616 
617   /// DataSize - The data size of the record being laid out.
618   uint64_t DataSize;
619 
620   CharUnits NonVirtualSize;
621   CharUnits NonVirtualAlignment;
622 
623   /// PrimaryBase - the primary base class (if one exists) of the class
624   /// we're laying out.
625   const CXXRecordDecl *PrimaryBase;
626 
627   /// PrimaryBaseIsVirtual - Whether the primary base of the class we're laying
628   /// out is virtual.
629   bool PrimaryBaseIsVirtual;
630 
631   /// HasOwnVFPtr - Whether the class provides its own vtable/vftbl
632   /// pointer, as opposed to inheriting one from a primary base class.
633   bool HasOwnVFPtr;
634 
635   typedef llvm::DenseMap<const CXXRecordDecl *, CharUnits> BaseOffsetsMapTy;
636 
637   /// Bases - base classes and their offsets in the record.
638   BaseOffsetsMapTy Bases;
639 
640   // VBases - virtual base classes and their offsets in the record.
641   ASTRecordLayout::VBaseOffsetsMapTy VBases;
642 
643   /// IndirectPrimaryBases - Virtual base classes, direct or indirect, that are
644   /// primary base classes for some other direct or indirect base class.
645   CXXIndirectPrimaryBaseSet IndirectPrimaryBases;
646 
647   /// FirstNearlyEmptyVBase - The first nearly empty virtual base class in
648   /// inheritance graph order. Used for determining the primary base class.
649   const CXXRecordDecl *FirstNearlyEmptyVBase;
650 
651   /// VisitedVirtualBases - A set of all the visited virtual bases, used to
652   /// avoid visiting virtual bases more than once.
653   llvm::SmallPtrSet<const CXXRecordDecl *, 4> VisitedVirtualBases;
654 
655   /// Valid if UseExternalLayout is true.
656   ExternalLayout External;
657 
658   ItaniumRecordLayoutBuilder(const ASTContext &Context,
659                              EmptySubobjectMap *EmptySubobjects)
660       : Context(Context), EmptySubobjects(EmptySubobjects), Size(0),
661         Alignment(CharUnits::One()), UnpackedAlignment(CharUnits::One()),
662         UseExternalLayout(false), InferAlignment(false), Packed(false),
663         IsUnion(false), IsMac68kAlign(false), IsMsStruct(false),
664         UnfilledBitsInLastUnit(0), LastBitfieldTypeSize(0),
665         MaxFieldAlignment(CharUnits::Zero()), DataSize(0),
666         NonVirtualSize(CharUnits::Zero()),
667         NonVirtualAlignment(CharUnits::One()), PrimaryBase(nullptr),
668         PrimaryBaseIsVirtual(false), HasOwnVFPtr(false),
669         FirstNearlyEmptyVBase(nullptr) {}
670 
671   void Layout(const RecordDecl *D);
672   void Layout(const CXXRecordDecl *D);
673   void Layout(const ObjCInterfaceDecl *D);
674 
675   void LayoutFields(const RecordDecl *D);
676   void LayoutField(const FieldDecl *D, bool InsertExtraPadding);
677   void LayoutWideBitField(uint64_t FieldSize, uint64_t TypeSize,
678                           bool FieldPacked, const FieldDecl *D);
679   void LayoutBitField(const FieldDecl *D);
680 
681   TargetCXXABI getCXXABI() const {
682     return Context.getTargetInfo().getCXXABI();
683   }
684 
685   /// BaseSubobjectInfoAllocator - Allocator for BaseSubobjectInfo objects.
686   llvm::SpecificBumpPtrAllocator<BaseSubobjectInfo> BaseSubobjectInfoAllocator;
687 
688   typedef llvm::DenseMap<const CXXRecordDecl *, BaseSubobjectInfo *>
689     BaseSubobjectInfoMapTy;
690 
691   /// VirtualBaseInfo - Map from all the (direct or indirect) virtual bases
692   /// of the class we're laying out to their base subobject info.
693   BaseSubobjectInfoMapTy VirtualBaseInfo;
694 
695   /// NonVirtualBaseInfo - Map from all the direct non-virtual bases of the
696   /// class we're laying out to their base subobject info.
697   BaseSubobjectInfoMapTy NonVirtualBaseInfo;
698 
699   /// ComputeBaseSubobjectInfo - Compute the base subobject information for the
700   /// bases of the given class.
701   void ComputeBaseSubobjectInfo(const CXXRecordDecl *RD);
702 
703   /// ComputeBaseSubobjectInfo - Compute the base subobject information for a
704   /// single class and all of its base classes.
705   BaseSubobjectInfo *ComputeBaseSubobjectInfo(const CXXRecordDecl *RD,
706                                               bool IsVirtual,
707                                               BaseSubobjectInfo *Derived);
708 
709   /// DeterminePrimaryBase - Determine the primary base of the given class.
710   void DeterminePrimaryBase(const CXXRecordDecl *RD);
711 
712   void SelectPrimaryVBase(const CXXRecordDecl *RD);
713 
714   void EnsureVTablePointerAlignment(CharUnits UnpackedBaseAlign);
715 
716   /// LayoutNonVirtualBases - Determines the primary base class (if any) and
717   /// lays it out. Will then proceed to lay out all non-virtual base clasess.
718   void LayoutNonVirtualBases(const CXXRecordDecl *RD);
719 
720   /// LayoutNonVirtualBase - Lays out a single non-virtual base.
721   void LayoutNonVirtualBase(const BaseSubobjectInfo *Base);
722 
723   void AddPrimaryVirtualBaseOffsets(const BaseSubobjectInfo *Info,
724                                     CharUnits Offset);
725 
726   /// LayoutVirtualBases - Lays out all the virtual bases.
727   void LayoutVirtualBases(const CXXRecordDecl *RD,
728                           const CXXRecordDecl *MostDerivedClass);
729 
730   /// LayoutVirtualBase - Lays out a single virtual base.
731   void LayoutVirtualBase(const BaseSubobjectInfo *Base);
732 
733   /// LayoutBase - Will lay out a base and return the offset where it was
734   /// placed, in chars.
735   CharUnits LayoutBase(const BaseSubobjectInfo *Base);
736 
737   /// InitializeLayout - Initialize record layout for the given record decl.
738   void InitializeLayout(const Decl *D);
739 
740   /// FinishLayout - Finalize record layout. Adjust record size based on the
741   /// alignment.
742   void FinishLayout(const NamedDecl *D);
743 
744   void UpdateAlignment(CharUnits NewAlignment, CharUnits UnpackedNewAlignment);
745   void UpdateAlignment(CharUnits NewAlignment) {
746     UpdateAlignment(NewAlignment, NewAlignment);
747   }
748 
749   /// \brief Retrieve the externally-supplied field offset for the given
750   /// field.
751   ///
752   /// \param Field The field whose offset is being queried.
753   /// \param ComputedOffset The offset that we've computed for this field.
754   uint64_t updateExternalFieldOffset(const FieldDecl *Field,
755                                      uint64_t ComputedOffset);
756 
757   void CheckFieldPadding(uint64_t Offset, uint64_t UnpaddedOffset,
758                           uint64_t UnpackedOffset, unsigned UnpackedAlign,
759                           bool isPacked, const FieldDecl *D);
760 
761   DiagnosticBuilder Diag(SourceLocation Loc, unsigned DiagID);
762 
763   CharUnits getSize() const {
764     assert(Size % Context.getCharWidth() == 0);
765     return Context.toCharUnitsFromBits(Size);
766   }
767   uint64_t getSizeInBits() const { return Size; }
768 
769   void setSize(CharUnits NewSize) { Size = Context.toBits(NewSize); }
770   void setSize(uint64_t NewSize) { Size = NewSize; }
771 
772   CharUnits getAligment() const { return Alignment; }
773 
774   CharUnits getDataSize() const {
775     assert(DataSize % Context.getCharWidth() == 0);
776     return Context.toCharUnitsFromBits(DataSize);
777   }
778   uint64_t getDataSizeInBits() const { return DataSize; }
779 
780   void setDataSize(CharUnits NewSize) { DataSize = Context.toBits(NewSize); }
781   void setDataSize(uint64_t NewSize) { DataSize = NewSize; }
782 
783   ItaniumRecordLayoutBuilder(const ItaniumRecordLayoutBuilder &) = delete;
784   void operator=(const ItaniumRecordLayoutBuilder &) = delete;
785 };
786 } // end anonymous namespace
787 
788 void ItaniumRecordLayoutBuilder::SelectPrimaryVBase(const CXXRecordDecl *RD) {
789   for (const auto &I : RD->bases()) {
790     assert(!I.getType()->isDependentType() &&
791            "Cannot layout class with dependent bases.");
792 
793     const CXXRecordDecl *Base = I.getType()->getAsCXXRecordDecl();
794 
795     // Check if this is a nearly empty virtual base.
796     if (I.isVirtual() && Context.isNearlyEmpty(Base)) {
797       // If it's not an indirect primary base, then we've found our primary
798       // base.
799       if (!IndirectPrimaryBases.count(Base)) {
800         PrimaryBase = Base;
801         PrimaryBaseIsVirtual = true;
802         return;
803       }
804 
805       // Is this the first nearly empty virtual base?
806       if (!FirstNearlyEmptyVBase)
807         FirstNearlyEmptyVBase = Base;
808     }
809 
810     SelectPrimaryVBase(Base);
811     if (PrimaryBase)
812       return;
813   }
814 }
815 
816 /// DeterminePrimaryBase - Determine the primary base of the given class.
817 void ItaniumRecordLayoutBuilder::DeterminePrimaryBase(const CXXRecordDecl *RD) {
818   // If the class isn't dynamic, it won't have a primary base.
819   if (!RD->isDynamicClass())
820     return;
821 
822   // Compute all the primary virtual bases for all of our direct and
823   // indirect bases, and record all their primary virtual base classes.
824   RD->getIndirectPrimaryBases(IndirectPrimaryBases);
825 
826   // If the record has a dynamic base class, attempt to choose a primary base
827   // class. It is the first (in direct base class order) non-virtual dynamic
828   // base class, if one exists.
829   for (const auto &I : RD->bases()) {
830     // Ignore virtual bases.
831     if (I.isVirtual())
832       continue;
833 
834     const CXXRecordDecl *Base = I.getType()->getAsCXXRecordDecl();
835 
836     if (Base->isDynamicClass()) {
837       // We found it.
838       PrimaryBase = Base;
839       PrimaryBaseIsVirtual = false;
840       return;
841     }
842   }
843 
844   // Under the Itanium ABI, if there is no non-virtual primary base class,
845   // try to compute the primary virtual base.  The primary virtual base is
846   // the first nearly empty virtual base that is not an indirect primary
847   // virtual base class, if one exists.
848   if (RD->getNumVBases() != 0) {
849     SelectPrimaryVBase(RD);
850     if (PrimaryBase)
851       return;
852   }
853 
854   // Otherwise, it is the first indirect primary base class, if one exists.
855   if (FirstNearlyEmptyVBase) {
856     PrimaryBase = FirstNearlyEmptyVBase;
857     PrimaryBaseIsVirtual = true;
858     return;
859   }
860 
861   assert(!PrimaryBase && "Should not get here with a primary base!");
862 }
863 
864 BaseSubobjectInfo *ItaniumRecordLayoutBuilder::ComputeBaseSubobjectInfo(
865     const CXXRecordDecl *RD, bool IsVirtual, BaseSubobjectInfo *Derived) {
866   BaseSubobjectInfo *Info;
867 
868   if (IsVirtual) {
869     // Check if we already have info about this virtual base.
870     BaseSubobjectInfo *&InfoSlot = VirtualBaseInfo[RD];
871     if (InfoSlot) {
872       assert(InfoSlot->Class == RD && "Wrong class for virtual base info!");
873       return InfoSlot;
874     }
875 
876     // We don't, create it.
877     InfoSlot = new (BaseSubobjectInfoAllocator.Allocate()) BaseSubobjectInfo;
878     Info = InfoSlot;
879   } else {
880     Info = new (BaseSubobjectInfoAllocator.Allocate()) BaseSubobjectInfo;
881   }
882 
883   Info->Class = RD;
884   Info->IsVirtual = IsVirtual;
885   Info->Derived = nullptr;
886   Info->PrimaryVirtualBaseInfo = nullptr;
887 
888   const CXXRecordDecl *PrimaryVirtualBase = nullptr;
889   BaseSubobjectInfo *PrimaryVirtualBaseInfo = nullptr;
890 
891   // Check if this base has a primary virtual base.
892   if (RD->getNumVBases()) {
893     const ASTRecordLayout &Layout = Context.getASTRecordLayout(RD);
894     if (Layout.isPrimaryBaseVirtual()) {
895       // This base does have a primary virtual base.
896       PrimaryVirtualBase = Layout.getPrimaryBase();
897       assert(PrimaryVirtualBase && "Didn't have a primary virtual base!");
898 
899       // Now check if we have base subobject info about this primary base.
900       PrimaryVirtualBaseInfo = VirtualBaseInfo.lookup(PrimaryVirtualBase);
901 
902       if (PrimaryVirtualBaseInfo) {
903         if (PrimaryVirtualBaseInfo->Derived) {
904           // We did have info about this primary base, and it turns out that it
905           // has already been claimed as a primary virtual base for another
906           // base.
907           PrimaryVirtualBase = nullptr;
908         } else {
909           // We can claim this base as our primary base.
910           Info->PrimaryVirtualBaseInfo = PrimaryVirtualBaseInfo;
911           PrimaryVirtualBaseInfo->Derived = Info;
912         }
913       }
914     }
915   }
916 
917   // Now go through all direct bases.
918   for (const auto &I : RD->bases()) {
919     bool IsVirtual = I.isVirtual();
920 
921     const CXXRecordDecl *BaseDecl = I.getType()->getAsCXXRecordDecl();
922 
923     Info->Bases.push_back(ComputeBaseSubobjectInfo(BaseDecl, IsVirtual, Info));
924   }
925 
926   if (PrimaryVirtualBase && !PrimaryVirtualBaseInfo) {
927     // Traversing the bases must have created the base info for our primary
928     // virtual base.
929     PrimaryVirtualBaseInfo = VirtualBaseInfo.lookup(PrimaryVirtualBase);
930     assert(PrimaryVirtualBaseInfo &&
931            "Did not create a primary virtual base!");
932 
933     // Claim the primary virtual base as our primary virtual base.
934     Info->PrimaryVirtualBaseInfo = PrimaryVirtualBaseInfo;
935     PrimaryVirtualBaseInfo->Derived = Info;
936   }
937 
938   return Info;
939 }
940 
941 void ItaniumRecordLayoutBuilder::ComputeBaseSubobjectInfo(
942     const CXXRecordDecl *RD) {
943   for (const auto &I : RD->bases()) {
944     bool IsVirtual = I.isVirtual();
945 
946     const CXXRecordDecl *BaseDecl = I.getType()->getAsCXXRecordDecl();
947 
948     // Compute the base subobject info for this base.
949     BaseSubobjectInfo *Info = ComputeBaseSubobjectInfo(BaseDecl, IsVirtual,
950                                                        nullptr);
951 
952     if (IsVirtual) {
953       // ComputeBaseInfo has already added this base for us.
954       assert(VirtualBaseInfo.count(BaseDecl) &&
955              "Did not add virtual base!");
956     } else {
957       // Add the base info to the map of non-virtual bases.
958       assert(!NonVirtualBaseInfo.count(BaseDecl) &&
959              "Non-virtual base already exists!");
960       NonVirtualBaseInfo.insert(std::make_pair(BaseDecl, Info));
961     }
962   }
963 }
964 
965 void ItaniumRecordLayoutBuilder::EnsureVTablePointerAlignment(
966     CharUnits UnpackedBaseAlign) {
967   CharUnits BaseAlign = (Packed) ? CharUnits::One() : UnpackedBaseAlign;
968 
969   // The maximum field alignment overrides base align.
970   if (!MaxFieldAlignment.isZero()) {
971     BaseAlign = std::min(BaseAlign, MaxFieldAlignment);
972     UnpackedBaseAlign = std::min(UnpackedBaseAlign, MaxFieldAlignment);
973   }
974 
975   // Round up the current record size to pointer alignment.
976   setSize(getSize().alignTo(BaseAlign));
977   setDataSize(getSize());
978 
979   // Update the alignment.
980   UpdateAlignment(BaseAlign, UnpackedBaseAlign);
981 }
982 
983 void ItaniumRecordLayoutBuilder::LayoutNonVirtualBases(
984     const CXXRecordDecl *RD) {
985   // Then, determine the primary base class.
986   DeterminePrimaryBase(RD);
987 
988   // Compute base subobject info.
989   ComputeBaseSubobjectInfo(RD);
990 
991   // If we have a primary base class, lay it out.
992   if (PrimaryBase) {
993     if (PrimaryBaseIsVirtual) {
994       // If the primary virtual base was a primary virtual base of some other
995       // base class we'll have to steal it.
996       BaseSubobjectInfo *PrimaryBaseInfo = VirtualBaseInfo.lookup(PrimaryBase);
997       PrimaryBaseInfo->Derived = nullptr;
998 
999       // We have a virtual primary base, insert it as an indirect primary base.
1000       IndirectPrimaryBases.insert(PrimaryBase);
1001 
1002       assert(!VisitedVirtualBases.count(PrimaryBase) &&
1003              "vbase already visited!");
1004       VisitedVirtualBases.insert(PrimaryBase);
1005 
1006       LayoutVirtualBase(PrimaryBaseInfo);
1007     } else {
1008       BaseSubobjectInfo *PrimaryBaseInfo =
1009         NonVirtualBaseInfo.lookup(PrimaryBase);
1010       assert(PrimaryBaseInfo &&
1011              "Did not find base info for non-virtual primary base!");
1012 
1013       LayoutNonVirtualBase(PrimaryBaseInfo);
1014     }
1015 
1016   // If this class needs a vtable/vf-table and didn't get one from a
1017   // primary base, add it in now.
1018   } else if (RD->isDynamicClass()) {
1019     assert(DataSize == 0 && "Vtable pointer must be at offset zero!");
1020     CharUnits PtrWidth =
1021       Context.toCharUnitsFromBits(Context.getTargetInfo().getPointerWidth(0));
1022     CharUnits PtrAlign =
1023       Context.toCharUnitsFromBits(Context.getTargetInfo().getPointerAlign(0));
1024     EnsureVTablePointerAlignment(PtrAlign);
1025     HasOwnVFPtr = true;
1026     setSize(getSize() + PtrWidth);
1027     setDataSize(getSize());
1028   }
1029 
1030   // Now lay out the non-virtual bases.
1031   for (const auto &I : RD->bases()) {
1032 
1033     // Ignore virtual bases.
1034     if (I.isVirtual())
1035       continue;
1036 
1037     const CXXRecordDecl *BaseDecl = I.getType()->getAsCXXRecordDecl();
1038 
1039     // Skip the primary base, because we've already laid it out.  The
1040     // !PrimaryBaseIsVirtual check is required because we might have a
1041     // non-virtual base of the same type as a primary virtual base.
1042     if (BaseDecl == PrimaryBase && !PrimaryBaseIsVirtual)
1043       continue;
1044 
1045     // Lay out the base.
1046     BaseSubobjectInfo *BaseInfo = NonVirtualBaseInfo.lookup(BaseDecl);
1047     assert(BaseInfo && "Did not find base info for non-virtual base!");
1048 
1049     LayoutNonVirtualBase(BaseInfo);
1050   }
1051 }
1052 
1053 void ItaniumRecordLayoutBuilder::LayoutNonVirtualBase(
1054     const BaseSubobjectInfo *Base) {
1055   // Layout the base.
1056   CharUnits Offset = LayoutBase(Base);
1057 
1058   // Add its base class offset.
1059   assert(!Bases.count(Base->Class) && "base offset already exists!");
1060   Bases.insert(std::make_pair(Base->Class, Offset));
1061 
1062   AddPrimaryVirtualBaseOffsets(Base, Offset);
1063 }
1064 
1065 void ItaniumRecordLayoutBuilder::AddPrimaryVirtualBaseOffsets(
1066     const BaseSubobjectInfo *Info, CharUnits Offset) {
1067   // This base isn't interesting, it has no virtual bases.
1068   if (!Info->Class->getNumVBases())
1069     return;
1070 
1071   // First, check if we have a virtual primary base to add offsets for.
1072   if (Info->PrimaryVirtualBaseInfo) {
1073     assert(Info->PrimaryVirtualBaseInfo->IsVirtual &&
1074            "Primary virtual base is not virtual!");
1075     if (Info->PrimaryVirtualBaseInfo->Derived == Info) {
1076       // Add the offset.
1077       assert(!VBases.count(Info->PrimaryVirtualBaseInfo->Class) &&
1078              "primary vbase offset already exists!");
1079       VBases.insert(std::make_pair(Info->PrimaryVirtualBaseInfo->Class,
1080                                    ASTRecordLayout::VBaseInfo(Offset, false)));
1081 
1082       // Traverse the primary virtual base.
1083       AddPrimaryVirtualBaseOffsets(Info->PrimaryVirtualBaseInfo, Offset);
1084     }
1085   }
1086 
1087   // Now go through all direct non-virtual bases.
1088   const ASTRecordLayout &Layout = Context.getASTRecordLayout(Info->Class);
1089   for (const BaseSubobjectInfo *Base : Info->Bases) {
1090     if (Base->IsVirtual)
1091       continue;
1092 
1093     CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base->Class);
1094     AddPrimaryVirtualBaseOffsets(Base, BaseOffset);
1095   }
1096 }
1097 
1098 void ItaniumRecordLayoutBuilder::LayoutVirtualBases(
1099     const CXXRecordDecl *RD, const CXXRecordDecl *MostDerivedClass) {
1100   const CXXRecordDecl *PrimaryBase;
1101   bool PrimaryBaseIsVirtual;
1102 
1103   if (MostDerivedClass == RD) {
1104     PrimaryBase = this->PrimaryBase;
1105     PrimaryBaseIsVirtual = this->PrimaryBaseIsVirtual;
1106   } else {
1107     const ASTRecordLayout &Layout = Context.getASTRecordLayout(RD);
1108     PrimaryBase = Layout.getPrimaryBase();
1109     PrimaryBaseIsVirtual = Layout.isPrimaryBaseVirtual();
1110   }
1111 
1112   for (const CXXBaseSpecifier &Base : RD->bases()) {
1113     assert(!Base.getType()->isDependentType() &&
1114            "Cannot layout class with dependent bases.");
1115 
1116     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
1117 
1118     if (Base.isVirtual()) {
1119       if (PrimaryBase != BaseDecl || !PrimaryBaseIsVirtual) {
1120         bool IndirectPrimaryBase = IndirectPrimaryBases.count(BaseDecl);
1121 
1122         // Only lay out the virtual base if it's not an indirect primary base.
1123         if (!IndirectPrimaryBase) {
1124           // Only visit virtual bases once.
1125           if (!VisitedVirtualBases.insert(BaseDecl).second)
1126             continue;
1127 
1128           const BaseSubobjectInfo *BaseInfo = VirtualBaseInfo.lookup(BaseDecl);
1129           assert(BaseInfo && "Did not find virtual base info!");
1130           LayoutVirtualBase(BaseInfo);
1131         }
1132       }
1133     }
1134 
1135     if (!BaseDecl->getNumVBases()) {
1136       // This base isn't interesting since it doesn't have any virtual bases.
1137       continue;
1138     }
1139 
1140     LayoutVirtualBases(BaseDecl, MostDerivedClass);
1141   }
1142 }
1143 
1144 void ItaniumRecordLayoutBuilder::LayoutVirtualBase(
1145     const BaseSubobjectInfo *Base) {
1146   assert(!Base->Derived && "Trying to lay out a primary virtual base!");
1147 
1148   // Layout the base.
1149   CharUnits Offset = LayoutBase(Base);
1150 
1151   // Add its base class offset.
1152   assert(!VBases.count(Base->Class) && "vbase offset already exists!");
1153   VBases.insert(std::make_pair(Base->Class,
1154                        ASTRecordLayout::VBaseInfo(Offset, false)));
1155 
1156   AddPrimaryVirtualBaseOffsets(Base, Offset);
1157 }
1158 
1159 CharUnits
1160 ItaniumRecordLayoutBuilder::LayoutBase(const BaseSubobjectInfo *Base) {
1161   const ASTRecordLayout &Layout = Context.getASTRecordLayout(Base->Class);
1162 
1163 
1164   CharUnits Offset;
1165 
1166   // Query the external layout to see if it provides an offset.
1167   bool HasExternalLayout = false;
1168   if (UseExternalLayout) {
1169     llvm::DenseMap<const CXXRecordDecl *, CharUnits>::iterator Known;
1170     if (Base->IsVirtual)
1171       HasExternalLayout = External.getExternalNVBaseOffset(Base->Class, Offset);
1172     else
1173       HasExternalLayout = External.getExternalVBaseOffset(Base->Class, Offset);
1174   }
1175 
1176   CharUnits UnpackedBaseAlign = Layout.getNonVirtualAlignment();
1177   CharUnits BaseAlign = (Packed) ? CharUnits::One() : UnpackedBaseAlign;
1178 
1179   // If we have an empty base class, try to place it at offset 0.
1180   if (Base->Class->isEmpty() &&
1181       (!HasExternalLayout || Offset == CharUnits::Zero()) &&
1182       EmptySubobjects->CanPlaceBaseAtOffset(Base, CharUnits::Zero())) {
1183     setSize(std::max(getSize(), Layout.getSize()));
1184     UpdateAlignment(BaseAlign, UnpackedBaseAlign);
1185 
1186     return CharUnits::Zero();
1187   }
1188 
1189   // The maximum field alignment overrides base align.
1190   if (!MaxFieldAlignment.isZero()) {
1191     BaseAlign = std::min(BaseAlign, MaxFieldAlignment);
1192     UnpackedBaseAlign = std::min(UnpackedBaseAlign, MaxFieldAlignment);
1193   }
1194 
1195   if (!HasExternalLayout) {
1196     // Round up the current record size to the base's alignment boundary.
1197     Offset = getDataSize().alignTo(BaseAlign);
1198 
1199     // Try to place the base.
1200     while (!EmptySubobjects->CanPlaceBaseAtOffset(Base, Offset))
1201       Offset += BaseAlign;
1202   } else {
1203     bool Allowed = EmptySubobjects->CanPlaceBaseAtOffset(Base, Offset);
1204     (void)Allowed;
1205     assert(Allowed && "Base subobject externally placed at overlapping offset");
1206 
1207     if (InferAlignment && Offset < getDataSize().alignTo(BaseAlign)) {
1208       // The externally-supplied base offset is before the base offset we
1209       // computed. Assume that the structure is packed.
1210       Alignment = CharUnits::One();
1211       InferAlignment = false;
1212     }
1213   }
1214 
1215   if (!Base->Class->isEmpty()) {
1216     // Update the data size.
1217     setDataSize(Offset + Layout.getNonVirtualSize());
1218 
1219     setSize(std::max(getSize(), getDataSize()));
1220   } else
1221     setSize(std::max(getSize(), Offset + Layout.getSize()));
1222 
1223   // Remember max struct/class alignment.
1224   UpdateAlignment(BaseAlign, UnpackedBaseAlign);
1225 
1226   return Offset;
1227 }
1228 
1229 void ItaniumRecordLayoutBuilder::InitializeLayout(const Decl *D) {
1230   if (const RecordDecl *RD = dyn_cast<RecordDecl>(D)) {
1231     IsUnion = RD->isUnion();
1232     IsMsStruct = RD->isMsStruct(Context);
1233   }
1234 
1235   Packed = D->hasAttr<PackedAttr>();
1236 
1237   // Honor the default struct packing maximum alignment flag.
1238   if (unsigned DefaultMaxFieldAlignment = Context.getLangOpts().PackStruct) {
1239     MaxFieldAlignment = CharUnits::fromQuantity(DefaultMaxFieldAlignment);
1240   }
1241 
1242   // mac68k alignment supersedes maximum field alignment and attribute aligned,
1243   // and forces all structures to have 2-byte alignment. The IBM docs on it
1244   // allude to additional (more complicated) semantics, especially with regard
1245   // to bit-fields, but gcc appears not to follow that.
1246   if (D->hasAttr<AlignMac68kAttr>()) {
1247     IsMac68kAlign = true;
1248     MaxFieldAlignment = CharUnits::fromQuantity(2);
1249     Alignment = CharUnits::fromQuantity(2);
1250   } else {
1251     if (const MaxFieldAlignmentAttr *MFAA = D->getAttr<MaxFieldAlignmentAttr>())
1252       MaxFieldAlignment = Context.toCharUnitsFromBits(MFAA->getAlignment());
1253 
1254     if (unsigned MaxAlign = D->getMaxAlignment())
1255       UpdateAlignment(Context.toCharUnitsFromBits(MaxAlign));
1256   }
1257 
1258   // If there is an external AST source, ask it for the various offsets.
1259   if (const RecordDecl *RD = dyn_cast<RecordDecl>(D))
1260     if (ExternalASTSource *Source = Context.getExternalSource()) {
1261       UseExternalLayout = Source->layoutRecordType(
1262           RD, External.Size, External.Align, External.FieldOffsets,
1263           External.BaseOffsets, External.VirtualBaseOffsets);
1264 
1265       // Update based on external alignment.
1266       if (UseExternalLayout) {
1267         if (External.Align > 0) {
1268           Alignment = Context.toCharUnitsFromBits(External.Align);
1269         } else {
1270           // The external source didn't have alignment information; infer it.
1271           InferAlignment = true;
1272         }
1273       }
1274     }
1275 }
1276 
1277 void ItaniumRecordLayoutBuilder::Layout(const RecordDecl *D) {
1278   InitializeLayout(D);
1279   LayoutFields(D);
1280 
1281   // Finally, round the size of the total struct up to the alignment of the
1282   // struct itself.
1283   FinishLayout(D);
1284 }
1285 
1286 void ItaniumRecordLayoutBuilder::Layout(const CXXRecordDecl *RD) {
1287   InitializeLayout(RD);
1288 
1289   // Lay out the vtable and the non-virtual bases.
1290   LayoutNonVirtualBases(RD);
1291 
1292   LayoutFields(RD);
1293 
1294   NonVirtualSize = Context.toCharUnitsFromBits(
1295       llvm::alignTo(getSizeInBits(), Context.getTargetInfo().getCharAlign()));
1296   NonVirtualAlignment = Alignment;
1297 
1298   // Lay out the virtual bases and add the primary virtual base offsets.
1299   LayoutVirtualBases(RD, RD);
1300 
1301   // Finally, round the size of the total struct up to the alignment
1302   // of the struct itself.
1303   FinishLayout(RD);
1304 
1305 #ifndef NDEBUG
1306   // Check that we have base offsets for all bases.
1307   for (const CXXBaseSpecifier &Base : RD->bases()) {
1308     if (Base.isVirtual())
1309       continue;
1310 
1311     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
1312 
1313     assert(Bases.count(BaseDecl) && "Did not find base offset!");
1314   }
1315 
1316   // And all virtual bases.
1317   for (const CXXBaseSpecifier &Base : RD->vbases()) {
1318     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
1319 
1320     assert(VBases.count(BaseDecl) && "Did not find base offset!");
1321   }
1322 #endif
1323 }
1324 
1325 void ItaniumRecordLayoutBuilder::Layout(const ObjCInterfaceDecl *D) {
1326   if (ObjCInterfaceDecl *SD = D->getSuperClass()) {
1327     const ASTRecordLayout &SL = Context.getASTObjCInterfaceLayout(SD);
1328 
1329     UpdateAlignment(SL.getAlignment());
1330 
1331     // We start laying out ivars not at the end of the superclass
1332     // structure, but at the next byte following the last field.
1333     setSize(SL.getDataSize());
1334     setDataSize(getSize());
1335   }
1336 
1337   InitializeLayout(D);
1338   // Layout each ivar sequentially.
1339   for (const ObjCIvarDecl *IVD = D->all_declared_ivar_begin(); IVD;
1340        IVD = IVD->getNextIvar())
1341     LayoutField(IVD, false);
1342 
1343   // Finally, round the size of the total struct up to the alignment of the
1344   // struct itself.
1345   FinishLayout(D);
1346 }
1347 
1348 void ItaniumRecordLayoutBuilder::LayoutFields(const RecordDecl *D) {
1349   // Layout each field, for now, just sequentially, respecting alignment.  In
1350   // the future, this will need to be tweakable by targets.
1351   bool InsertExtraPadding = D->mayInsertExtraPadding(/*EmitRemark=*/true);
1352   bool HasFlexibleArrayMember = D->hasFlexibleArrayMember();
1353   for (auto I = D->field_begin(), End = D->field_end(); I != End; ++I) {
1354     auto Next(I);
1355     ++Next;
1356     LayoutField(*I,
1357                 InsertExtraPadding && (Next != End || !HasFlexibleArrayMember));
1358   }
1359 }
1360 
1361 // Rounds the specified size to have it a multiple of the char size.
1362 static uint64_t
1363 roundUpSizeToCharAlignment(uint64_t Size,
1364                            const ASTContext &Context) {
1365   uint64_t CharAlignment = Context.getTargetInfo().getCharAlign();
1366   return llvm::alignTo(Size, CharAlignment);
1367 }
1368 
1369 void ItaniumRecordLayoutBuilder::LayoutWideBitField(uint64_t FieldSize,
1370                                                     uint64_t TypeSize,
1371                                                     bool FieldPacked,
1372                                                     const FieldDecl *D) {
1373   assert(Context.getLangOpts().CPlusPlus &&
1374          "Can only have wide bit-fields in C++!");
1375 
1376   // Itanium C++ ABI 2.4:
1377   //   If sizeof(T)*8 < n, let T' be the largest integral POD type with
1378   //   sizeof(T')*8 <= n.
1379 
1380   QualType IntegralPODTypes[] = {
1381     Context.UnsignedCharTy, Context.UnsignedShortTy, Context.UnsignedIntTy,
1382     Context.UnsignedLongTy, Context.UnsignedLongLongTy
1383   };
1384 
1385   QualType Type;
1386   for (const QualType &QT : IntegralPODTypes) {
1387     uint64_t Size = Context.getTypeSize(QT);
1388 
1389     if (Size > FieldSize)
1390       break;
1391 
1392     Type = QT;
1393   }
1394   assert(!Type.isNull() && "Did not find a type!");
1395 
1396   CharUnits TypeAlign = Context.getTypeAlignInChars(Type);
1397 
1398   // We're not going to use any of the unfilled bits in the last byte.
1399   UnfilledBitsInLastUnit = 0;
1400   LastBitfieldTypeSize = 0;
1401 
1402   uint64_t FieldOffset;
1403   uint64_t UnpaddedFieldOffset = getDataSizeInBits() - UnfilledBitsInLastUnit;
1404 
1405   if (IsUnion) {
1406     uint64_t RoundedFieldSize = roundUpSizeToCharAlignment(FieldSize,
1407                                                            Context);
1408     setDataSize(std::max(getDataSizeInBits(), RoundedFieldSize));
1409     FieldOffset = 0;
1410   } else {
1411     // The bitfield is allocated starting at the next offset aligned
1412     // appropriately for T', with length n bits.
1413     FieldOffset = llvm::alignTo(getDataSizeInBits(), Context.toBits(TypeAlign));
1414 
1415     uint64_t NewSizeInBits = FieldOffset + FieldSize;
1416 
1417     setDataSize(
1418         llvm::alignTo(NewSizeInBits, Context.getTargetInfo().getCharAlign()));
1419     UnfilledBitsInLastUnit = getDataSizeInBits() - NewSizeInBits;
1420   }
1421 
1422   // Place this field at the current location.
1423   FieldOffsets.push_back(FieldOffset);
1424 
1425   CheckFieldPadding(FieldOffset, UnpaddedFieldOffset, FieldOffset,
1426                     Context.toBits(TypeAlign), FieldPacked, D);
1427 
1428   // Update the size.
1429   setSize(std::max(getSizeInBits(), getDataSizeInBits()));
1430 
1431   // Remember max struct/class alignment.
1432   UpdateAlignment(TypeAlign);
1433 }
1434 
1435 void ItaniumRecordLayoutBuilder::LayoutBitField(const FieldDecl *D) {
1436   bool FieldPacked = Packed || D->hasAttr<PackedAttr>();
1437   uint64_t FieldSize = D->getBitWidthValue(Context);
1438   TypeInfo FieldInfo = Context.getTypeInfo(D->getType());
1439   uint64_t TypeSize = FieldInfo.Width;
1440   unsigned FieldAlign = FieldInfo.Align;
1441 
1442   // UnfilledBitsInLastUnit is the difference between the end of the
1443   // last allocated bitfield (i.e. the first bit offset available for
1444   // bitfields) and the end of the current data size in bits (i.e. the
1445   // first bit offset available for non-bitfields).  The current data
1446   // size in bits is always a multiple of the char size; additionally,
1447   // for ms_struct records it's also a multiple of the
1448   // LastBitfieldTypeSize (if set).
1449 
1450   // The struct-layout algorithm is dictated by the platform ABI,
1451   // which in principle could use almost any rules it likes.  In
1452   // practice, UNIXy targets tend to inherit the algorithm described
1453   // in the System V generic ABI.  The basic bitfield layout rule in
1454   // System V is to place bitfields at the next available bit offset
1455   // where the entire bitfield would fit in an aligned storage unit of
1456   // the declared type; it's okay if an earlier or later non-bitfield
1457   // is allocated in the same storage unit.  However, some targets
1458   // (those that !useBitFieldTypeAlignment(), e.g. ARM APCS) don't
1459   // require this storage unit to be aligned, and therefore always put
1460   // the bitfield at the next available bit offset.
1461 
1462   // ms_struct basically requests a complete replacement of the
1463   // platform ABI's struct-layout algorithm, with the high-level goal
1464   // of duplicating MSVC's layout.  For non-bitfields, this follows
1465   // the standard algorithm.  The basic bitfield layout rule is to
1466   // allocate an entire unit of the bitfield's declared type
1467   // (e.g. 'unsigned long'), then parcel it up among successive
1468   // bitfields whose declared types have the same size, making a new
1469   // unit as soon as the last can no longer store the whole value.
1470   // Since it completely replaces the platform ABI's algorithm,
1471   // settings like !useBitFieldTypeAlignment() do not apply.
1472 
1473   // A zero-width bitfield forces the use of a new storage unit for
1474   // later bitfields.  In general, this occurs by rounding up the
1475   // current size of the struct as if the algorithm were about to
1476   // place a non-bitfield of the field's formal type.  Usually this
1477   // does not change the alignment of the struct itself, but it does
1478   // on some targets (those that useZeroLengthBitfieldAlignment(),
1479   // e.g. ARM).  In ms_struct layout, zero-width bitfields are
1480   // ignored unless they follow a non-zero-width bitfield.
1481 
1482   // A field alignment restriction (e.g. from #pragma pack) or
1483   // specification (e.g. from __attribute__((aligned))) changes the
1484   // formal alignment of the field.  For System V, this alters the
1485   // required alignment of the notional storage unit that must contain
1486   // the bitfield.  For ms_struct, this only affects the placement of
1487   // new storage units.  In both cases, the effect of #pragma pack is
1488   // ignored on zero-width bitfields.
1489 
1490   // On System V, a packed field (e.g. from #pragma pack or
1491   // __attribute__((packed))) always uses the next available bit
1492   // offset.
1493 
1494   // In an ms_struct struct, the alignment of a fundamental type is
1495   // always equal to its size.  This is necessary in order to mimic
1496   // the i386 alignment rules on targets which might not fully align
1497   // all types (e.g. Darwin PPC32, where alignof(long long) == 4).
1498 
1499   // First, some simple bookkeeping to perform for ms_struct structs.
1500   if (IsMsStruct) {
1501     // The field alignment for integer types is always the size.
1502     FieldAlign = TypeSize;
1503 
1504     // If the previous field was not a bitfield, or was a bitfield
1505     // with a different storage unit size, we're done with that
1506     // storage unit.
1507     if (LastBitfieldTypeSize != TypeSize) {
1508       // Also, ignore zero-length bitfields after non-bitfields.
1509       if (!LastBitfieldTypeSize && !FieldSize)
1510         FieldAlign = 1;
1511 
1512       UnfilledBitsInLastUnit = 0;
1513       LastBitfieldTypeSize = 0;
1514     }
1515   }
1516 
1517   // If the field is wider than its declared type, it follows
1518   // different rules in all cases.
1519   if (FieldSize > TypeSize) {
1520     LayoutWideBitField(FieldSize, TypeSize, FieldPacked, D);
1521     return;
1522   }
1523 
1524   // Compute the next available bit offset.
1525   uint64_t FieldOffset =
1526     IsUnion ? 0 : (getDataSizeInBits() - UnfilledBitsInLastUnit);
1527 
1528   // Handle targets that don't honor bitfield type alignment.
1529   if (!IsMsStruct && !Context.getTargetInfo().useBitFieldTypeAlignment()) {
1530     // Some such targets do honor it on zero-width bitfields.
1531     if (FieldSize == 0 &&
1532         Context.getTargetInfo().useZeroLengthBitfieldAlignment()) {
1533       // The alignment to round up to is the max of the field's natural
1534       // alignment and a target-specific fixed value (sometimes zero).
1535       unsigned ZeroLengthBitfieldBoundary =
1536         Context.getTargetInfo().getZeroLengthBitfieldBoundary();
1537       FieldAlign = std::max(FieldAlign, ZeroLengthBitfieldBoundary);
1538 
1539     // If that doesn't apply, just ignore the field alignment.
1540     } else {
1541       FieldAlign = 1;
1542     }
1543   }
1544 
1545   // Remember the alignment we would have used if the field were not packed.
1546   unsigned UnpackedFieldAlign = FieldAlign;
1547 
1548   // Ignore the field alignment if the field is packed unless it has zero-size.
1549   if (!IsMsStruct && FieldPacked && FieldSize != 0)
1550     FieldAlign = 1;
1551 
1552   // But, if there's an 'aligned' attribute on the field, honor that.
1553   unsigned ExplicitFieldAlign = D->getMaxAlignment();
1554   if (ExplicitFieldAlign) {
1555     FieldAlign = std::max(FieldAlign, ExplicitFieldAlign);
1556     UnpackedFieldAlign = std::max(UnpackedFieldAlign, ExplicitFieldAlign);
1557   }
1558 
1559   // But, if there's a #pragma pack in play, that takes precedent over
1560   // even the 'aligned' attribute, for non-zero-width bitfields.
1561   if (!MaxFieldAlignment.isZero() && FieldSize) {
1562     unsigned MaxFieldAlignmentInBits = Context.toBits(MaxFieldAlignment);
1563     FieldAlign = std::min(FieldAlign, MaxFieldAlignmentInBits);
1564     UnpackedFieldAlign = std::min(UnpackedFieldAlign, MaxFieldAlignmentInBits);
1565   }
1566 
1567   // But, ms_struct just ignores all of that in unions, even explicit
1568   // alignment attributes.
1569   if (IsMsStruct && IsUnion) {
1570     FieldAlign = UnpackedFieldAlign = 1;
1571   }
1572 
1573   // For purposes of diagnostics, we're going to simultaneously
1574   // compute the field offsets that we would have used if we weren't
1575   // adding any alignment padding or if the field weren't packed.
1576   uint64_t UnpaddedFieldOffset = FieldOffset;
1577   uint64_t UnpackedFieldOffset = FieldOffset;
1578 
1579   // Check if we need to add padding to fit the bitfield within an
1580   // allocation unit with the right size and alignment.  The rules are
1581   // somewhat different here for ms_struct structs.
1582   if (IsMsStruct) {
1583     // If it's not a zero-width bitfield, and we can fit the bitfield
1584     // into the active storage unit (and we haven't already decided to
1585     // start a new storage unit), just do so, regardless of any other
1586     // other consideration.  Otherwise, round up to the right alignment.
1587     if (FieldSize == 0 || FieldSize > UnfilledBitsInLastUnit) {
1588       FieldOffset = llvm::alignTo(FieldOffset, FieldAlign);
1589       UnpackedFieldOffset =
1590           llvm::alignTo(UnpackedFieldOffset, UnpackedFieldAlign);
1591       UnfilledBitsInLastUnit = 0;
1592     }
1593 
1594   } else {
1595     // #pragma pack, with any value, suppresses the insertion of padding.
1596     bool AllowPadding = MaxFieldAlignment.isZero();
1597 
1598     // Compute the real offset.
1599     if (FieldSize == 0 ||
1600         (AllowPadding &&
1601          (FieldOffset & (FieldAlign-1)) + FieldSize > TypeSize)) {
1602       FieldOffset = llvm::alignTo(FieldOffset, FieldAlign);
1603     } else if (ExplicitFieldAlign) {
1604       // TODO: figure it out what needs to be done on targets that don't honor
1605       // bit-field type alignment like ARM APCS ABI.
1606       FieldOffset = llvm::alignTo(FieldOffset, ExplicitFieldAlign);
1607     }
1608 
1609     // Repeat the computation for diagnostic purposes.
1610     if (FieldSize == 0 ||
1611         (AllowPadding &&
1612          (UnpackedFieldOffset & (UnpackedFieldAlign-1)) + FieldSize > TypeSize))
1613       UnpackedFieldOffset =
1614           llvm::alignTo(UnpackedFieldOffset, UnpackedFieldAlign);
1615     else if (ExplicitFieldAlign)
1616       UnpackedFieldOffset =
1617           llvm::alignTo(UnpackedFieldOffset, ExplicitFieldAlign);
1618   }
1619 
1620   // If we're using external layout, give the external layout a chance
1621   // to override this information.
1622   if (UseExternalLayout)
1623     FieldOffset = updateExternalFieldOffset(D, FieldOffset);
1624 
1625   // Okay, place the bitfield at the calculated offset.
1626   FieldOffsets.push_back(FieldOffset);
1627 
1628   // Bookkeeping:
1629 
1630   // Anonymous members don't affect the overall record alignment,
1631   // except on targets where they do.
1632   if (!IsMsStruct &&
1633       !Context.getTargetInfo().useZeroLengthBitfieldAlignment() &&
1634       !D->getIdentifier())
1635     FieldAlign = UnpackedFieldAlign = 1;
1636 
1637   // Diagnose differences in layout due to padding or packing.
1638   if (!UseExternalLayout)
1639     CheckFieldPadding(FieldOffset, UnpaddedFieldOffset, UnpackedFieldOffset,
1640                       UnpackedFieldAlign, FieldPacked, D);
1641 
1642   // Update DataSize to include the last byte containing (part of) the bitfield.
1643 
1644   // For unions, this is just a max operation, as usual.
1645   if (IsUnion) {
1646     // For ms_struct, allocate the entire storage unit --- unless this
1647     // is a zero-width bitfield, in which case just use a size of 1.
1648     uint64_t RoundedFieldSize;
1649     if (IsMsStruct) {
1650       RoundedFieldSize =
1651         (FieldSize ? TypeSize : Context.getTargetInfo().getCharWidth());
1652 
1653     // Otherwise, allocate just the number of bytes required to store
1654     // the bitfield.
1655     } else {
1656       RoundedFieldSize = roundUpSizeToCharAlignment(FieldSize, Context);
1657     }
1658     setDataSize(std::max(getDataSizeInBits(), RoundedFieldSize));
1659 
1660   // For non-zero-width bitfields in ms_struct structs, allocate a new
1661   // storage unit if necessary.
1662   } else if (IsMsStruct && FieldSize) {
1663     // We should have cleared UnfilledBitsInLastUnit in every case
1664     // where we changed storage units.
1665     if (!UnfilledBitsInLastUnit) {
1666       setDataSize(FieldOffset + TypeSize);
1667       UnfilledBitsInLastUnit = TypeSize;
1668     }
1669     UnfilledBitsInLastUnit -= FieldSize;
1670     LastBitfieldTypeSize = TypeSize;
1671 
1672   // Otherwise, bump the data size up to include the bitfield,
1673   // including padding up to char alignment, and then remember how
1674   // bits we didn't use.
1675   } else {
1676     uint64_t NewSizeInBits = FieldOffset + FieldSize;
1677     uint64_t CharAlignment = Context.getTargetInfo().getCharAlign();
1678     setDataSize(llvm::alignTo(NewSizeInBits, CharAlignment));
1679     UnfilledBitsInLastUnit = getDataSizeInBits() - NewSizeInBits;
1680 
1681     // The only time we can get here for an ms_struct is if this is a
1682     // zero-width bitfield, which doesn't count as anything for the
1683     // purposes of unfilled bits.
1684     LastBitfieldTypeSize = 0;
1685   }
1686 
1687   // Update the size.
1688   setSize(std::max(getSizeInBits(), getDataSizeInBits()));
1689 
1690   // Remember max struct/class alignment.
1691   UpdateAlignment(Context.toCharUnitsFromBits(FieldAlign),
1692                   Context.toCharUnitsFromBits(UnpackedFieldAlign));
1693 }
1694 
1695 void ItaniumRecordLayoutBuilder::LayoutField(const FieldDecl *D,
1696                                              bool InsertExtraPadding) {
1697   if (D->isBitField()) {
1698     LayoutBitField(D);
1699     return;
1700   }
1701 
1702   uint64_t UnpaddedFieldOffset = getDataSizeInBits() - UnfilledBitsInLastUnit;
1703 
1704   // Reset the unfilled bits.
1705   UnfilledBitsInLastUnit = 0;
1706   LastBitfieldTypeSize = 0;
1707 
1708   bool FieldPacked = Packed || D->hasAttr<PackedAttr>();
1709   CharUnits FieldOffset =
1710     IsUnion ? CharUnits::Zero() : getDataSize();
1711   CharUnits FieldSize;
1712   CharUnits FieldAlign;
1713 
1714   if (D->getType()->isIncompleteArrayType()) {
1715     // This is a flexible array member; we can't directly
1716     // query getTypeInfo about these, so we figure it out here.
1717     // Flexible array members don't have any size, but they
1718     // have to be aligned appropriately for their element type.
1719     FieldSize = CharUnits::Zero();
1720     const ArrayType* ATy = Context.getAsArrayType(D->getType());
1721     FieldAlign = Context.getTypeAlignInChars(ATy->getElementType());
1722   } else if (const ReferenceType *RT = D->getType()->getAs<ReferenceType>()) {
1723     unsigned AS = RT->getPointeeType().getAddressSpace();
1724     FieldSize =
1725       Context.toCharUnitsFromBits(Context.getTargetInfo().getPointerWidth(AS));
1726     FieldAlign =
1727       Context.toCharUnitsFromBits(Context.getTargetInfo().getPointerAlign(AS));
1728   } else {
1729     std::pair<CharUnits, CharUnits> FieldInfo =
1730       Context.getTypeInfoInChars(D->getType());
1731     FieldSize = FieldInfo.first;
1732     FieldAlign = FieldInfo.second;
1733 
1734     if (IsMsStruct) {
1735       // If MS bitfield layout is required, figure out what type is being
1736       // laid out and align the field to the width of that type.
1737 
1738       // Resolve all typedefs down to their base type and round up the field
1739       // alignment if necessary.
1740       QualType T = Context.getBaseElementType(D->getType());
1741       if (const BuiltinType *BTy = T->getAs<BuiltinType>()) {
1742         CharUnits TypeSize = Context.getTypeSizeInChars(BTy);
1743         if (TypeSize > FieldAlign)
1744           FieldAlign = TypeSize;
1745       }
1746     }
1747   }
1748 
1749   // The align if the field is not packed. This is to check if the attribute
1750   // was unnecessary (-Wpacked).
1751   CharUnits UnpackedFieldAlign = FieldAlign;
1752   CharUnits UnpackedFieldOffset = FieldOffset;
1753 
1754   if (FieldPacked)
1755     FieldAlign = CharUnits::One();
1756   CharUnits MaxAlignmentInChars =
1757     Context.toCharUnitsFromBits(D->getMaxAlignment());
1758   FieldAlign = std::max(FieldAlign, MaxAlignmentInChars);
1759   UnpackedFieldAlign = std::max(UnpackedFieldAlign, MaxAlignmentInChars);
1760 
1761   // The maximum field alignment overrides the aligned attribute.
1762   if (!MaxFieldAlignment.isZero()) {
1763     FieldAlign = std::min(FieldAlign, MaxFieldAlignment);
1764     UnpackedFieldAlign = std::min(UnpackedFieldAlign, MaxFieldAlignment);
1765   }
1766 
1767   // Round up the current record size to the field's alignment boundary.
1768   FieldOffset = FieldOffset.alignTo(FieldAlign);
1769   UnpackedFieldOffset = UnpackedFieldOffset.alignTo(UnpackedFieldAlign);
1770 
1771   if (UseExternalLayout) {
1772     FieldOffset = Context.toCharUnitsFromBits(
1773                     updateExternalFieldOffset(D, Context.toBits(FieldOffset)));
1774 
1775     if (!IsUnion && EmptySubobjects) {
1776       // Record the fact that we're placing a field at this offset.
1777       bool Allowed = EmptySubobjects->CanPlaceFieldAtOffset(D, FieldOffset);
1778       (void)Allowed;
1779       assert(Allowed && "Externally-placed field cannot be placed here");
1780     }
1781   } else {
1782     if (!IsUnion && EmptySubobjects) {
1783       // Check if we can place the field at this offset.
1784       while (!EmptySubobjects->CanPlaceFieldAtOffset(D, FieldOffset)) {
1785         // We couldn't place the field at the offset. Try again at a new offset.
1786         FieldOffset += FieldAlign;
1787       }
1788     }
1789   }
1790 
1791   // Place this field at the current location.
1792   FieldOffsets.push_back(Context.toBits(FieldOffset));
1793 
1794   if (!UseExternalLayout)
1795     CheckFieldPadding(Context.toBits(FieldOffset), UnpaddedFieldOffset,
1796                       Context.toBits(UnpackedFieldOffset),
1797                       Context.toBits(UnpackedFieldAlign), FieldPacked, D);
1798 
1799   if (InsertExtraPadding) {
1800     CharUnits ASanAlignment = CharUnits::fromQuantity(8);
1801     CharUnits ExtraSizeForAsan = ASanAlignment;
1802     if (FieldSize % ASanAlignment)
1803       ExtraSizeForAsan +=
1804           ASanAlignment - CharUnits::fromQuantity(FieldSize % ASanAlignment);
1805     FieldSize += ExtraSizeForAsan;
1806   }
1807 
1808   // Reserve space for this field.
1809   uint64_t FieldSizeInBits = Context.toBits(FieldSize);
1810   if (IsUnion)
1811     setDataSize(std::max(getDataSizeInBits(), FieldSizeInBits));
1812   else
1813     setDataSize(FieldOffset + FieldSize);
1814 
1815   // Update the size.
1816   setSize(std::max(getSizeInBits(), getDataSizeInBits()));
1817 
1818   // Remember max struct/class alignment.
1819   UpdateAlignment(FieldAlign, UnpackedFieldAlign);
1820 }
1821 
1822 void ItaniumRecordLayoutBuilder::FinishLayout(const NamedDecl *D) {
1823   // In C++, records cannot be of size 0.
1824   if (Context.getLangOpts().CPlusPlus && getSizeInBits() == 0) {
1825     if (const CXXRecordDecl *RD = dyn_cast<CXXRecordDecl>(D)) {
1826       // Compatibility with gcc requires a class (pod or non-pod)
1827       // which is not empty but of size 0; such as having fields of
1828       // array of zero-length, remains of Size 0
1829       if (RD->isEmpty())
1830         setSize(CharUnits::One());
1831     }
1832     else
1833       setSize(CharUnits::One());
1834   }
1835 
1836   // Finally, round the size of the record up to the alignment of the
1837   // record itself.
1838   uint64_t UnpaddedSize = getSizeInBits() - UnfilledBitsInLastUnit;
1839   uint64_t UnpackedSizeInBits =
1840       llvm::alignTo(getSizeInBits(), Context.toBits(UnpackedAlignment));
1841   CharUnits UnpackedSize = Context.toCharUnitsFromBits(UnpackedSizeInBits);
1842   uint64_t RoundedSize =
1843       llvm::alignTo(getSizeInBits(), Context.toBits(Alignment));
1844 
1845   if (UseExternalLayout) {
1846     // If we're inferring alignment, and the external size is smaller than
1847     // our size after we've rounded up to alignment, conservatively set the
1848     // alignment to 1.
1849     if (InferAlignment && External.Size < RoundedSize) {
1850       Alignment = CharUnits::One();
1851       InferAlignment = false;
1852     }
1853     setSize(External.Size);
1854     return;
1855   }
1856 
1857   // Set the size to the final size.
1858   setSize(RoundedSize);
1859 
1860   unsigned CharBitNum = Context.getTargetInfo().getCharWidth();
1861   if (const RecordDecl *RD = dyn_cast<RecordDecl>(D)) {
1862     // Warn if padding was introduced to the struct/class/union.
1863     if (getSizeInBits() > UnpaddedSize) {
1864       unsigned PadSize = getSizeInBits() - UnpaddedSize;
1865       bool InBits = true;
1866       if (PadSize % CharBitNum == 0) {
1867         PadSize = PadSize / CharBitNum;
1868         InBits = false;
1869       }
1870       Diag(RD->getLocation(), diag::warn_padded_struct_size)
1871           << Context.getTypeDeclType(RD)
1872           << PadSize
1873           << (InBits ? 1 : 0); // (byte|bit)
1874     }
1875 
1876     // Warn if we packed it unnecessarily. If the alignment is 1 byte don't
1877     // bother since there won't be alignment issues.
1878     if (Packed && UnpackedAlignment > CharUnits::One() &&
1879         getSize() == UnpackedSize)
1880       Diag(D->getLocation(), diag::warn_unnecessary_packed)
1881           << Context.getTypeDeclType(RD);
1882   }
1883 }
1884 
1885 void ItaniumRecordLayoutBuilder::UpdateAlignment(
1886     CharUnits NewAlignment, CharUnits UnpackedNewAlignment) {
1887   // The alignment is not modified when using 'mac68k' alignment or when
1888   // we have an externally-supplied layout that also provides overall alignment.
1889   if (IsMac68kAlign || (UseExternalLayout && !InferAlignment))
1890     return;
1891 
1892   if (NewAlignment > Alignment) {
1893     assert(llvm::isPowerOf2_64(NewAlignment.getQuantity()) &&
1894            "Alignment not a power of 2");
1895     Alignment = NewAlignment;
1896   }
1897 
1898   if (UnpackedNewAlignment > UnpackedAlignment) {
1899     assert(llvm::isPowerOf2_64(UnpackedNewAlignment.getQuantity()) &&
1900            "Alignment not a power of 2");
1901     UnpackedAlignment = UnpackedNewAlignment;
1902   }
1903 }
1904 
1905 uint64_t
1906 ItaniumRecordLayoutBuilder::updateExternalFieldOffset(const FieldDecl *Field,
1907                                                       uint64_t ComputedOffset) {
1908   uint64_t ExternalFieldOffset = External.getExternalFieldOffset(Field);
1909 
1910   if (InferAlignment && ExternalFieldOffset < ComputedOffset) {
1911     // The externally-supplied field offset is before the field offset we
1912     // computed. Assume that the structure is packed.
1913     Alignment = CharUnits::One();
1914     InferAlignment = false;
1915   }
1916 
1917   // Use the externally-supplied field offset.
1918   return ExternalFieldOffset;
1919 }
1920 
1921 /// \brief Get diagnostic %select index for tag kind for
1922 /// field padding diagnostic message.
1923 /// WARNING: Indexes apply to particular diagnostics only!
1924 ///
1925 /// \returns diagnostic %select index.
1926 static unsigned getPaddingDiagFromTagKind(TagTypeKind Tag) {
1927   switch (Tag) {
1928   case TTK_Struct: return 0;
1929   case TTK_Interface: return 1;
1930   case TTK_Class: return 2;
1931   default: llvm_unreachable("Invalid tag kind for field padding diagnostic!");
1932   }
1933 }
1934 
1935 void ItaniumRecordLayoutBuilder::CheckFieldPadding(
1936     uint64_t Offset, uint64_t UnpaddedOffset, uint64_t UnpackedOffset,
1937     unsigned UnpackedAlign, bool isPacked, const FieldDecl *D) {
1938   // We let objc ivars without warning, objc interfaces generally are not used
1939   // for padding tricks.
1940   if (isa<ObjCIvarDecl>(D))
1941     return;
1942 
1943   // Don't warn about structs created without a SourceLocation.  This can
1944   // be done by clients of the AST, such as codegen.
1945   if (D->getLocation().isInvalid())
1946     return;
1947 
1948   unsigned CharBitNum = Context.getTargetInfo().getCharWidth();
1949 
1950   // Warn if padding was introduced to the struct/class.
1951   if (!IsUnion && Offset > UnpaddedOffset) {
1952     unsigned PadSize = Offset - UnpaddedOffset;
1953     bool InBits = true;
1954     if (PadSize % CharBitNum == 0) {
1955       PadSize = PadSize / CharBitNum;
1956       InBits = false;
1957     }
1958     if (D->getIdentifier())
1959       Diag(D->getLocation(), diag::warn_padded_struct_field)
1960           << getPaddingDiagFromTagKind(D->getParent()->getTagKind())
1961           << Context.getTypeDeclType(D->getParent())
1962           << PadSize
1963           << (InBits ? 1 : 0) // (byte|bit)
1964           << D->getIdentifier();
1965     else
1966       Diag(D->getLocation(), diag::warn_padded_struct_anon_field)
1967           << getPaddingDiagFromTagKind(D->getParent()->getTagKind())
1968           << Context.getTypeDeclType(D->getParent())
1969           << PadSize
1970           << (InBits ? 1 : 0); // (byte|bit)
1971   }
1972 
1973   // Warn if we packed it unnecessarily. If the alignment is 1 byte don't
1974   // bother since there won't be alignment issues.
1975   if (isPacked && UnpackedAlign > CharBitNum && Offset == UnpackedOffset)
1976     Diag(D->getLocation(), diag::warn_unnecessary_packed)
1977         << D->getIdentifier();
1978 }
1979 
1980 static const CXXMethodDecl *computeKeyFunction(ASTContext &Context,
1981                                                const CXXRecordDecl *RD) {
1982   // If a class isn't polymorphic it doesn't have a key function.
1983   if (!RD->isPolymorphic())
1984     return nullptr;
1985 
1986   // A class that is not externally visible doesn't have a key function. (Or
1987   // at least, there's no point to assigning a key function to such a class;
1988   // this doesn't affect the ABI.)
1989   if (!RD->isExternallyVisible())
1990     return nullptr;
1991 
1992   // Template instantiations don't have key functions per Itanium C++ ABI 5.2.6.
1993   // Same behavior as GCC.
1994   TemplateSpecializationKind TSK = RD->getTemplateSpecializationKind();
1995   if (TSK == TSK_ImplicitInstantiation ||
1996       TSK == TSK_ExplicitInstantiationDeclaration ||
1997       TSK == TSK_ExplicitInstantiationDefinition)
1998     return nullptr;
1999 
2000   bool allowInlineFunctions =
2001     Context.getTargetInfo().getCXXABI().canKeyFunctionBeInline();
2002 
2003   for (const CXXMethodDecl *MD : RD->methods()) {
2004     if (!MD->isVirtual())
2005       continue;
2006 
2007     if (MD->isPure())
2008       continue;
2009 
2010     // Ignore implicit member functions, they are always marked as inline, but
2011     // they don't have a body until they're defined.
2012     if (MD->isImplicit())
2013       continue;
2014 
2015     if (MD->isInlineSpecified())
2016       continue;
2017 
2018     if (MD->hasInlineBody())
2019       continue;
2020 
2021     // Ignore inline deleted or defaulted functions.
2022     if (!MD->isUserProvided())
2023       continue;
2024 
2025     // In certain ABIs, ignore functions with out-of-line inline definitions.
2026     if (!allowInlineFunctions) {
2027       const FunctionDecl *Def;
2028       if (MD->hasBody(Def) && Def->isInlineSpecified())
2029         continue;
2030     }
2031 
2032     if (Context.getLangOpts().CUDA) {
2033       // While compiler may see key method in this TU, during CUDA
2034       // compilation we should ignore methods that are not accessible
2035       // on this side of compilation.
2036       if (Context.getLangOpts().CUDAIsDevice) {
2037         // In device mode ignore methods without __device__ attribute.
2038         if (!MD->hasAttr<CUDADeviceAttr>())
2039           continue;
2040       } else {
2041         // In host mode ignore __device__-only methods.
2042         if (!MD->hasAttr<CUDAHostAttr>() && MD->hasAttr<CUDADeviceAttr>())
2043           continue;
2044       }
2045     }
2046 
2047     // If the key function is dllimport but the class isn't, then the class has
2048     // no key function. The DLL that exports the key function won't export the
2049     // vtable in this case.
2050     if (MD->hasAttr<DLLImportAttr>() && !RD->hasAttr<DLLImportAttr>())
2051       return nullptr;
2052 
2053     // We found it.
2054     return MD;
2055   }
2056 
2057   return nullptr;
2058 }
2059 
2060 DiagnosticBuilder ItaniumRecordLayoutBuilder::Diag(SourceLocation Loc,
2061                                                    unsigned DiagID) {
2062   return Context.getDiagnostics().Report(Loc, DiagID);
2063 }
2064 
2065 /// Does the target C++ ABI require us to skip over the tail-padding
2066 /// of the given class (considering it as a base class) when allocating
2067 /// objects?
2068 static bool mustSkipTailPadding(TargetCXXABI ABI, const CXXRecordDecl *RD) {
2069   switch (ABI.getTailPaddingUseRules()) {
2070   case TargetCXXABI::AlwaysUseTailPadding:
2071     return false;
2072 
2073   case TargetCXXABI::UseTailPaddingUnlessPOD03:
2074     // FIXME: To the extent that this is meant to cover the Itanium ABI
2075     // rules, we should implement the restrictions about over-sized
2076     // bitfields:
2077     //
2078     // http://mentorembedded.github.com/cxx-abi/abi.html#POD :
2079     //   In general, a type is considered a POD for the purposes of
2080     //   layout if it is a POD type (in the sense of ISO C++
2081     //   [basic.types]). However, a POD-struct or POD-union (in the
2082     //   sense of ISO C++ [class]) with a bitfield member whose
2083     //   declared width is wider than the declared type of the
2084     //   bitfield is not a POD for the purpose of layout.  Similarly,
2085     //   an array type is not a POD for the purpose of layout if the
2086     //   element type of the array is not a POD for the purpose of
2087     //   layout.
2088     //
2089     //   Where references to the ISO C++ are made in this paragraph,
2090     //   the Technical Corrigendum 1 version of the standard is
2091     //   intended.
2092     return RD->isPOD();
2093 
2094   case TargetCXXABI::UseTailPaddingUnlessPOD11:
2095     // This is equivalent to RD->getTypeForDecl().isCXX11PODType(),
2096     // but with a lot of abstraction penalty stripped off.  This does
2097     // assume that these properties are set correctly even in C++98
2098     // mode; fortunately, that is true because we want to assign
2099     // consistently semantics to the type-traits intrinsics (or at
2100     // least as many of them as possible).
2101     return RD->isTrivial() && RD->isStandardLayout();
2102   }
2103 
2104   llvm_unreachable("bad tail-padding use kind");
2105 }
2106 
2107 static bool isMsLayout(const ASTContext &Context) {
2108   return Context.getTargetInfo().getCXXABI().isMicrosoft();
2109 }
2110 
2111 // This section contains an implementation of struct layout that is, up to the
2112 // included tests, compatible with cl.exe (2013).  The layout produced is
2113 // significantly different than those produced by the Itanium ABI.  Here we note
2114 // the most important differences.
2115 //
2116 // * The alignment of bitfields in unions is ignored when computing the
2117 //   alignment of the union.
2118 // * The existence of zero-width bitfield that occurs after anything other than
2119 //   a non-zero length bitfield is ignored.
2120 // * There is no explicit primary base for the purposes of layout.  All bases
2121 //   with vfptrs are laid out first, followed by all bases without vfptrs.
2122 // * The Itanium equivalent vtable pointers are split into a vfptr (virtual
2123 //   function pointer) and a vbptr (virtual base pointer).  They can each be
2124 //   shared with a, non-virtual bases. These bases need not be the same.  vfptrs
2125 //   always occur at offset 0.  vbptrs can occur at an arbitrary offset and are
2126 //   placed after the lexiographically last non-virtual base.  This placement
2127 //   is always before fields but can be in the middle of the non-virtual bases
2128 //   due to the two-pass layout scheme for non-virtual-bases.
2129 // * Virtual bases sometimes require a 'vtordisp' field that is laid out before
2130 //   the virtual base and is used in conjunction with virtual overrides during
2131 //   construction and destruction.  This is always a 4 byte value and is used as
2132 //   an alternative to constructor vtables.
2133 // * vtordisps are allocated in a block of memory with size and alignment equal
2134 //   to the alignment of the completed structure (before applying __declspec(
2135 //   align())).  The vtordisp always occur at the end of the allocation block,
2136 //   immediately prior to the virtual base.
2137 // * vfptrs are injected after all bases and fields have been laid out.  In
2138 //   order to guarantee proper alignment of all fields, the vfptr injection
2139 //   pushes all bases and fields back by the alignment imposed by those bases
2140 //   and fields.  This can potentially add a significant amount of padding.
2141 //   vfptrs are always injected at offset 0.
2142 // * vbptrs are injected after all bases and fields have been laid out.  In
2143 //   order to guarantee proper alignment of all fields, the vfptr injection
2144 //   pushes all bases and fields back by the alignment imposed by those bases
2145 //   and fields.  This can potentially add a significant amount of padding.
2146 //   vbptrs are injected immediately after the last non-virtual base as
2147 //   lexiographically ordered in the code.  If this site isn't pointer aligned
2148 //   the vbptr is placed at the next properly aligned location.  Enough padding
2149 //   is added to guarantee a fit.
2150 // * The last zero sized non-virtual base can be placed at the end of the
2151 //   struct (potentially aliasing another object), or may alias with the first
2152 //   field, even if they are of the same type.
2153 // * The last zero size virtual base may be placed at the end of the struct
2154 //   potentially aliasing another object.
2155 // * The ABI attempts to avoid aliasing of zero sized bases by adding padding
2156 //   between bases or vbases with specific properties.  The criteria for
2157 //   additional padding between two bases is that the first base is zero sized
2158 //   or ends with a zero sized subobject and the second base is zero sized or
2159 //   trails with a zero sized base or field (sharing of vfptrs can reorder the
2160 //   layout of the so the leading base is not always the first one declared).
2161 //   This rule does take into account fields that are not records, so padding
2162 //   will occur even if the last field is, e.g. an int. The padding added for
2163 //   bases is 1 byte.  The padding added between vbases depends on the alignment
2164 //   of the object but is at least 4 bytes (in both 32 and 64 bit modes).
2165 // * There is no concept of non-virtual alignment, non-virtual alignment and
2166 //   alignment are always identical.
2167 // * There is a distinction between alignment and required alignment.
2168 //   __declspec(align) changes the required alignment of a struct.  This
2169 //   alignment is _always_ obeyed, even in the presence of #pragma pack. A
2170 //   record inherits required alignment from all of its fields and bases.
2171 // * __declspec(align) on bitfields has the effect of changing the bitfield's
2172 //   alignment instead of its required alignment.  This is the only known way
2173 //   to make the alignment of a struct bigger than 8.  Interestingly enough
2174 //   this alignment is also immune to the effects of #pragma pack and can be
2175 //   used to create structures with large alignment under #pragma pack.
2176 //   However, because it does not impact required alignment, such a structure,
2177 //   when used as a field or base, will not be aligned if #pragma pack is
2178 //   still active at the time of use.
2179 //
2180 // Known incompatibilities:
2181 // * all: #pragma pack between fields in a record
2182 // * 2010 and back: If the last field in a record is a bitfield, every object
2183 //   laid out after the record will have extra padding inserted before it.  The
2184 //   extra padding will have size equal to the size of the storage class of the
2185 //   bitfield.  0 sized bitfields don't exhibit this behavior and the extra
2186 //   padding can be avoided by adding a 0 sized bitfield after the non-zero-
2187 //   sized bitfield.
2188 // * 2012 and back: In 64-bit mode, if the alignment of a record is 16 or
2189 //   greater due to __declspec(align()) then a second layout phase occurs after
2190 //   The locations of the vf and vb pointers are known.  This layout phase
2191 //   suffers from the "last field is a bitfield" bug in 2010 and results in
2192 //   _every_ field getting padding put in front of it, potentially including the
2193 //   vfptr, leaving the vfprt at a non-zero location which results in a fault if
2194 //   anything tries to read the vftbl.  The second layout phase also treats
2195 //   bitfields as separate entities and gives them each storage rather than
2196 //   packing them.  Additionally, because this phase appears to perform a
2197 //   (an unstable) sort on the members before laying them out and because merged
2198 //   bitfields have the same address, the bitfields end up in whatever order
2199 //   the sort left them in, a behavior we could never hope to replicate.
2200 
2201 namespace {
2202 struct MicrosoftRecordLayoutBuilder {
2203   struct ElementInfo {
2204     CharUnits Size;
2205     CharUnits Alignment;
2206   };
2207   typedef llvm::DenseMap<const CXXRecordDecl *, CharUnits> BaseOffsetsMapTy;
2208   MicrosoftRecordLayoutBuilder(const ASTContext &Context) : Context(Context) {}
2209 private:
2210   MicrosoftRecordLayoutBuilder(const MicrosoftRecordLayoutBuilder &) = delete;
2211   void operator=(const MicrosoftRecordLayoutBuilder &) = delete;
2212 public:
2213   void layout(const RecordDecl *RD);
2214   void cxxLayout(const CXXRecordDecl *RD);
2215   /// \brief Initializes size and alignment and honors some flags.
2216   void initializeLayout(const RecordDecl *RD);
2217   /// \brief Initialized C++ layout, compute alignment and virtual alignment and
2218   /// existence of vfptrs and vbptrs.  Alignment is needed before the vfptr is
2219   /// laid out.
2220   void initializeCXXLayout(const CXXRecordDecl *RD);
2221   void layoutNonVirtualBases(const CXXRecordDecl *RD);
2222   void layoutNonVirtualBase(const CXXRecordDecl *BaseDecl,
2223                             const ASTRecordLayout &BaseLayout,
2224                             const ASTRecordLayout *&PreviousBaseLayout);
2225   void injectVFPtr(const CXXRecordDecl *RD);
2226   void injectVBPtr(const CXXRecordDecl *RD);
2227   /// \brief Lays out the fields of the record.  Also rounds size up to
2228   /// alignment.
2229   void layoutFields(const RecordDecl *RD);
2230   void layoutField(const FieldDecl *FD);
2231   void layoutBitField(const FieldDecl *FD);
2232   /// \brief Lays out a single zero-width bit-field in the record and handles
2233   /// special cases associated with zero-width bit-fields.
2234   void layoutZeroWidthBitField(const FieldDecl *FD);
2235   void layoutVirtualBases(const CXXRecordDecl *RD);
2236   void finalizeLayout(const RecordDecl *RD);
2237   /// \brief Gets the size and alignment of a base taking pragma pack and
2238   /// __declspec(align) into account.
2239   ElementInfo getAdjustedElementInfo(const ASTRecordLayout &Layout);
2240   /// \brief Gets the size and alignment of a field taking pragma  pack and
2241   /// __declspec(align) into account.  It also updates RequiredAlignment as a
2242   /// side effect because it is most convenient to do so here.
2243   ElementInfo getAdjustedElementInfo(const FieldDecl *FD);
2244   /// \brief Places a field at an offset in CharUnits.
2245   void placeFieldAtOffset(CharUnits FieldOffset) {
2246     FieldOffsets.push_back(Context.toBits(FieldOffset));
2247   }
2248   /// \brief Places a bitfield at a bit offset.
2249   void placeFieldAtBitOffset(uint64_t FieldOffset) {
2250     FieldOffsets.push_back(FieldOffset);
2251   }
2252   /// \brief Compute the set of virtual bases for which vtordisps are required.
2253   void computeVtorDispSet(
2254       llvm::SmallPtrSetImpl<const CXXRecordDecl *> &HasVtorDispSet,
2255       const CXXRecordDecl *RD) const;
2256   const ASTContext &Context;
2257   /// \brief The size of the record being laid out.
2258   CharUnits Size;
2259   /// \brief The non-virtual size of the record layout.
2260   CharUnits NonVirtualSize;
2261   /// \brief The data size of the record layout.
2262   CharUnits DataSize;
2263   /// \brief The current alignment of the record layout.
2264   CharUnits Alignment;
2265   /// \brief The maximum allowed field alignment. This is set by #pragma pack.
2266   CharUnits MaxFieldAlignment;
2267   /// \brief The alignment that this record must obey.  This is imposed by
2268   /// __declspec(align()) on the record itself or one of its fields or bases.
2269   CharUnits RequiredAlignment;
2270   /// \brief The size of the allocation of the currently active bitfield.
2271   /// This value isn't meaningful unless LastFieldIsNonZeroWidthBitfield
2272   /// is true.
2273   CharUnits CurrentBitfieldSize;
2274   /// \brief Offset to the virtual base table pointer (if one exists).
2275   CharUnits VBPtrOffset;
2276   /// \brief Minimum record size possible.
2277   CharUnits MinEmptyStructSize;
2278   /// \brief The size and alignment info of a pointer.
2279   ElementInfo PointerInfo;
2280   /// \brief The primary base class (if one exists).
2281   const CXXRecordDecl *PrimaryBase;
2282   /// \brief The class we share our vb-pointer with.
2283   const CXXRecordDecl *SharedVBPtrBase;
2284   /// \brief The collection of field offsets.
2285   SmallVector<uint64_t, 16> FieldOffsets;
2286   /// \brief Base classes and their offsets in the record.
2287   BaseOffsetsMapTy Bases;
2288   /// \brief virtual base classes and their offsets in the record.
2289   ASTRecordLayout::VBaseOffsetsMapTy VBases;
2290   /// \brief The number of remaining bits in our last bitfield allocation.
2291   /// This value isn't meaningful unless LastFieldIsNonZeroWidthBitfield is
2292   /// true.
2293   unsigned RemainingBitsInField;
2294   bool IsUnion : 1;
2295   /// \brief True if the last field laid out was a bitfield and was not 0
2296   /// width.
2297   bool LastFieldIsNonZeroWidthBitfield : 1;
2298   /// \brief True if the class has its own vftable pointer.
2299   bool HasOwnVFPtr : 1;
2300   /// \brief True if the class has a vbtable pointer.
2301   bool HasVBPtr : 1;
2302   /// \brief True if the last sub-object within the type is zero sized or the
2303   /// object itself is zero sized.  This *does not* count members that are not
2304   /// records.  Only used for MS-ABI.
2305   bool EndsWithZeroSizedObject : 1;
2306   /// \brief True if this class is zero sized or first base is zero sized or
2307   /// has this property.  Only used for MS-ABI.
2308   bool LeadsWithZeroSizedBase : 1;
2309 
2310   /// \brief True if the external AST source provided a layout for this record.
2311   bool UseExternalLayout : 1;
2312 
2313   /// \brief The layout provided by the external AST source. Only active if
2314   /// UseExternalLayout is true.
2315   ExternalLayout External;
2316 };
2317 } // namespace
2318 
2319 MicrosoftRecordLayoutBuilder::ElementInfo
2320 MicrosoftRecordLayoutBuilder::getAdjustedElementInfo(
2321     const ASTRecordLayout &Layout) {
2322   ElementInfo Info;
2323   Info.Alignment = Layout.getAlignment();
2324   // Respect pragma pack.
2325   if (!MaxFieldAlignment.isZero())
2326     Info.Alignment = std::min(Info.Alignment, MaxFieldAlignment);
2327   // Track zero-sized subobjects here where it's already available.
2328   EndsWithZeroSizedObject = Layout.hasZeroSizedSubObject();
2329   // Respect required alignment, this is necessary because we may have adjusted
2330   // the alignment in the case of pragam pack.  Note that the required alignment
2331   // doesn't actually apply to the struct alignment at this point.
2332   Alignment = std::max(Alignment, Info.Alignment);
2333   RequiredAlignment = std::max(RequiredAlignment, Layout.getRequiredAlignment());
2334   Info.Alignment = std::max(Info.Alignment, Layout.getRequiredAlignment());
2335   Info.Size = Layout.getNonVirtualSize();
2336   return Info;
2337 }
2338 
2339 MicrosoftRecordLayoutBuilder::ElementInfo
2340 MicrosoftRecordLayoutBuilder::getAdjustedElementInfo(
2341     const FieldDecl *FD) {
2342   // Get the alignment of the field type's natural alignment, ignore any
2343   // alignment attributes.
2344   ElementInfo Info;
2345   std::tie(Info.Size, Info.Alignment) =
2346       Context.getTypeInfoInChars(FD->getType()->getUnqualifiedDesugaredType());
2347   // Respect align attributes on the field.
2348   CharUnits FieldRequiredAlignment =
2349       Context.toCharUnitsFromBits(FD->getMaxAlignment());
2350   // Respect align attributes on the type.
2351   if (Context.isAlignmentRequired(FD->getType()))
2352     FieldRequiredAlignment = std::max(
2353         Context.getTypeAlignInChars(FD->getType()), FieldRequiredAlignment);
2354   // Respect attributes applied to subobjects of the field.
2355   if (FD->isBitField())
2356     // For some reason __declspec align impacts alignment rather than required
2357     // alignment when it is applied to bitfields.
2358     Info.Alignment = std::max(Info.Alignment, FieldRequiredAlignment);
2359   else {
2360     if (auto RT =
2361             FD->getType()->getBaseElementTypeUnsafe()->getAs<RecordType>()) {
2362       auto const &Layout = Context.getASTRecordLayout(RT->getDecl());
2363       EndsWithZeroSizedObject = Layout.hasZeroSizedSubObject();
2364       FieldRequiredAlignment = std::max(FieldRequiredAlignment,
2365                                         Layout.getRequiredAlignment());
2366     }
2367     // Capture required alignment as a side-effect.
2368     RequiredAlignment = std::max(RequiredAlignment, FieldRequiredAlignment);
2369   }
2370   // Respect pragma pack, attribute pack and declspec align
2371   if (!MaxFieldAlignment.isZero())
2372     Info.Alignment = std::min(Info.Alignment, MaxFieldAlignment);
2373   if (FD->hasAttr<PackedAttr>())
2374     Info.Alignment = CharUnits::One();
2375   Info.Alignment = std::max(Info.Alignment, FieldRequiredAlignment);
2376   return Info;
2377 }
2378 
2379 void MicrosoftRecordLayoutBuilder::layout(const RecordDecl *RD) {
2380   // For C record layout, zero-sized records always have size 4.
2381   MinEmptyStructSize = CharUnits::fromQuantity(4);
2382   initializeLayout(RD);
2383   layoutFields(RD);
2384   DataSize = Size = Size.alignTo(Alignment);
2385   RequiredAlignment = std::max(
2386       RequiredAlignment, Context.toCharUnitsFromBits(RD->getMaxAlignment()));
2387   finalizeLayout(RD);
2388 }
2389 
2390 void MicrosoftRecordLayoutBuilder::cxxLayout(const CXXRecordDecl *RD) {
2391   // The C++ standard says that empty structs have size 1.
2392   MinEmptyStructSize = CharUnits::One();
2393   initializeLayout(RD);
2394   initializeCXXLayout(RD);
2395   layoutNonVirtualBases(RD);
2396   layoutFields(RD);
2397   injectVBPtr(RD);
2398   injectVFPtr(RD);
2399   if (HasOwnVFPtr || (HasVBPtr && !SharedVBPtrBase))
2400     Alignment = std::max(Alignment, PointerInfo.Alignment);
2401   auto RoundingAlignment = Alignment;
2402   if (!MaxFieldAlignment.isZero())
2403     RoundingAlignment = std::min(RoundingAlignment, MaxFieldAlignment);
2404   NonVirtualSize = Size = Size.alignTo(RoundingAlignment);
2405   RequiredAlignment = std::max(
2406       RequiredAlignment, Context.toCharUnitsFromBits(RD->getMaxAlignment()));
2407   layoutVirtualBases(RD);
2408   finalizeLayout(RD);
2409 }
2410 
2411 void MicrosoftRecordLayoutBuilder::initializeLayout(const RecordDecl *RD) {
2412   IsUnion = RD->isUnion();
2413   Size = CharUnits::Zero();
2414   Alignment = CharUnits::One();
2415   // In 64-bit mode we always perform an alignment step after laying out vbases.
2416   // In 32-bit mode we do not.  The check to see if we need to perform alignment
2417   // checks the RequiredAlignment field and performs alignment if it isn't 0.
2418   RequiredAlignment = Context.getTargetInfo().getTriple().isArch64Bit()
2419                           ? CharUnits::One()
2420                           : CharUnits::Zero();
2421   // Compute the maximum field alignment.
2422   MaxFieldAlignment = CharUnits::Zero();
2423   // Honor the default struct packing maximum alignment flag.
2424   if (unsigned DefaultMaxFieldAlignment = Context.getLangOpts().PackStruct)
2425       MaxFieldAlignment = CharUnits::fromQuantity(DefaultMaxFieldAlignment);
2426   // Honor the packing attribute.  The MS-ABI ignores pragma pack if its larger
2427   // than the pointer size.
2428   if (const MaxFieldAlignmentAttr *MFAA = RD->getAttr<MaxFieldAlignmentAttr>()){
2429     unsigned PackedAlignment = MFAA->getAlignment();
2430     if (PackedAlignment <= Context.getTargetInfo().getPointerWidth(0))
2431       MaxFieldAlignment = Context.toCharUnitsFromBits(PackedAlignment);
2432   }
2433   // Packed attribute forces max field alignment to be 1.
2434   if (RD->hasAttr<PackedAttr>())
2435     MaxFieldAlignment = CharUnits::One();
2436 
2437   // Try to respect the external layout if present.
2438   UseExternalLayout = false;
2439   if (ExternalASTSource *Source = Context.getExternalSource())
2440     UseExternalLayout = Source->layoutRecordType(
2441         RD, External.Size, External.Align, External.FieldOffsets,
2442         External.BaseOffsets, External.VirtualBaseOffsets);
2443 }
2444 
2445 void
2446 MicrosoftRecordLayoutBuilder::initializeCXXLayout(const CXXRecordDecl *RD) {
2447   EndsWithZeroSizedObject = false;
2448   LeadsWithZeroSizedBase = false;
2449   HasOwnVFPtr = false;
2450   HasVBPtr = false;
2451   PrimaryBase = nullptr;
2452   SharedVBPtrBase = nullptr;
2453   // Calculate pointer size and alignment.  These are used for vfptr and vbprt
2454   // injection.
2455   PointerInfo.Size =
2456       Context.toCharUnitsFromBits(Context.getTargetInfo().getPointerWidth(0));
2457   PointerInfo.Alignment =
2458       Context.toCharUnitsFromBits(Context.getTargetInfo().getPointerAlign(0));
2459   // Respect pragma pack.
2460   if (!MaxFieldAlignment.isZero())
2461     PointerInfo.Alignment = std::min(PointerInfo.Alignment, MaxFieldAlignment);
2462 }
2463 
2464 void
2465 MicrosoftRecordLayoutBuilder::layoutNonVirtualBases(const CXXRecordDecl *RD) {
2466   // The MS-ABI lays out all bases that contain leading vfptrs before it lays
2467   // out any bases that do not contain vfptrs.  We implement this as two passes
2468   // over the bases.  This approach guarantees that the primary base is laid out
2469   // first.  We use these passes to calculate some additional aggregated
2470   // information about the bases, such as reqruied alignment and the presence of
2471   // zero sized members.
2472   const ASTRecordLayout *PreviousBaseLayout = nullptr;
2473   // Iterate through the bases and lay out the non-virtual ones.
2474   for (const CXXBaseSpecifier &Base : RD->bases()) {
2475     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
2476     const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(BaseDecl);
2477     // Mark and skip virtual bases.
2478     if (Base.isVirtual()) {
2479       HasVBPtr = true;
2480       continue;
2481     }
2482     // Check fo a base to share a VBPtr with.
2483     if (!SharedVBPtrBase && BaseLayout.hasVBPtr()) {
2484       SharedVBPtrBase = BaseDecl;
2485       HasVBPtr = true;
2486     }
2487     // Only lay out bases with extendable VFPtrs on the first pass.
2488     if (!BaseLayout.hasExtendableVFPtr())
2489       continue;
2490     // If we don't have a primary base, this one qualifies.
2491     if (!PrimaryBase) {
2492       PrimaryBase = BaseDecl;
2493       LeadsWithZeroSizedBase = BaseLayout.leadsWithZeroSizedBase();
2494     }
2495     // Lay out the base.
2496     layoutNonVirtualBase(BaseDecl, BaseLayout, PreviousBaseLayout);
2497   }
2498   // Figure out if we need a fresh VFPtr for this class.
2499   if (!PrimaryBase && RD->isDynamicClass())
2500     for (CXXRecordDecl::method_iterator i = RD->method_begin(),
2501                                         e = RD->method_end();
2502          !HasOwnVFPtr && i != e; ++i)
2503       HasOwnVFPtr = i->isVirtual() && i->size_overridden_methods() == 0;
2504   // If we don't have a primary base then we have a leading object that could
2505   // itself lead with a zero-sized object, something we track.
2506   bool CheckLeadingLayout = !PrimaryBase;
2507   // Iterate through the bases and lay out the non-virtual ones.
2508   for (const CXXBaseSpecifier &Base : RD->bases()) {
2509     if (Base.isVirtual())
2510       continue;
2511     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
2512     const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(BaseDecl);
2513     // Only lay out bases without extendable VFPtrs on the second pass.
2514     if (BaseLayout.hasExtendableVFPtr()) {
2515       VBPtrOffset = Bases[BaseDecl] + BaseLayout.getNonVirtualSize();
2516       continue;
2517     }
2518     // If this is the first layout, check to see if it leads with a zero sized
2519     // object.  If it does, so do we.
2520     if (CheckLeadingLayout) {
2521       CheckLeadingLayout = false;
2522       LeadsWithZeroSizedBase = BaseLayout.leadsWithZeroSizedBase();
2523     }
2524     // Lay out the base.
2525     layoutNonVirtualBase(BaseDecl, BaseLayout, PreviousBaseLayout);
2526     VBPtrOffset = Bases[BaseDecl] + BaseLayout.getNonVirtualSize();
2527   }
2528   // Set our VBPtroffset if we know it at this point.
2529   if (!HasVBPtr)
2530     VBPtrOffset = CharUnits::fromQuantity(-1);
2531   else if (SharedVBPtrBase) {
2532     const ASTRecordLayout &Layout = Context.getASTRecordLayout(SharedVBPtrBase);
2533     VBPtrOffset = Bases[SharedVBPtrBase] + Layout.getVBPtrOffset();
2534   }
2535 }
2536 
2537 void MicrosoftRecordLayoutBuilder::layoutNonVirtualBase(
2538     const CXXRecordDecl *BaseDecl,
2539     const ASTRecordLayout &BaseLayout,
2540     const ASTRecordLayout *&PreviousBaseLayout) {
2541   // Insert padding between two bases if the left first one is zero sized or
2542   // contains a zero sized subobject and the right is zero sized or one leads
2543   // with a zero sized base.
2544   if (PreviousBaseLayout && PreviousBaseLayout->hasZeroSizedSubObject() &&
2545       BaseLayout.leadsWithZeroSizedBase())
2546     Size++;
2547   ElementInfo Info = getAdjustedElementInfo(BaseLayout);
2548   CharUnits BaseOffset;
2549 
2550   // Respect the external AST source base offset, if present.
2551   bool FoundBase = false;
2552   if (UseExternalLayout) {
2553     FoundBase = External.getExternalNVBaseOffset(BaseDecl, BaseOffset);
2554     if (FoundBase)
2555       assert(BaseOffset >= Size && "base offset already allocated");
2556   }
2557 
2558   if (!FoundBase)
2559     BaseOffset = Size.alignTo(Info.Alignment);
2560   Bases.insert(std::make_pair(BaseDecl, BaseOffset));
2561   Size = BaseOffset + BaseLayout.getNonVirtualSize();
2562   PreviousBaseLayout = &BaseLayout;
2563 }
2564 
2565 void MicrosoftRecordLayoutBuilder::layoutFields(const RecordDecl *RD) {
2566   LastFieldIsNonZeroWidthBitfield = false;
2567   for (const FieldDecl *Field : RD->fields())
2568     layoutField(Field);
2569 }
2570 
2571 void MicrosoftRecordLayoutBuilder::layoutField(const FieldDecl *FD) {
2572   if (FD->isBitField()) {
2573     layoutBitField(FD);
2574     return;
2575   }
2576   LastFieldIsNonZeroWidthBitfield = false;
2577   ElementInfo Info = getAdjustedElementInfo(FD);
2578   Alignment = std::max(Alignment, Info.Alignment);
2579   if (IsUnion) {
2580     placeFieldAtOffset(CharUnits::Zero());
2581     Size = std::max(Size, Info.Size);
2582   } else {
2583     CharUnits FieldOffset;
2584     if (UseExternalLayout) {
2585       FieldOffset =
2586           Context.toCharUnitsFromBits(External.getExternalFieldOffset(FD));
2587       assert(FieldOffset >= Size && "field offset already allocated");
2588     } else {
2589       FieldOffset = Size.alignTo(Info.Alignment);
2590     }
2591     placeFieldAtOffset(FieldOffset);
2592     Size = FieldOffset + Info.Size;
2593   }
2594 }
2595 
2596 void MicrosoftRecordLayoutBuilder::layoutBitField(const FieldDecl *FD) {
2597   unsigned Width = FD->getBitWidthValue(Context);
2598   if (Width == 0) {
2599     layoutZeroWidthBitField(FD);
2600     return;
2601   }
2602   ElementInfo Info = getAdjustedElementInfo(FD);
2603   // Clamp the bitfield to a containable size for the sake of being able
2604   // to lay them out.  Sema will throw an error.
2605   if (Width > Context.toBits(Info.Size))
2606     Width = Context.toBits(Info.Size);
2607   // Check to see if this bitfield fits into an existing allocation.  Note:
2608   // MSVC refuses to pack bitfields of formal types with different sizes
2609   // into the same allocation.
2610   if (!IsUnion && LastFieldIsNonZeroWidthBitfield &&
2611       CurrentBitfieldSize == Info.Size && Width <= RemainingBitsInField) {
2612     placeFieldAtBitOffset(Context.toBits(Size) - RemainingBitsInField);
2613     RemainingBitsInField -= Width;
2614     return;
2615   }
2616   LastFieldIsNonZeroWidthBitfield = true;
2617   CurrentBitfieldSize = Info.Size;
2618   if (IsUnion) {
2619     placeFieldAtOffset(CharUnits::Zero());
2620     Size = std::max(Size, Info.Size);
2621     // TODO: Add a Sema warning that MS ignores bitfield alignment in unions.
2622   } else {
2623     // Allocate a new block of memory and place the bitfield in it.
2624     CharUnits FieldOffset = Size.alignTo(Info.Alignment);
2625     placeFieldAtOffset(FieldOffset);
2626     Size = FieldOffset + Info.Size;
2627     Alignment = std::max(Alignment, Info.Alignment);
2628     RemainingBitsInField = Context.toBits(Info.Size) - Width;
2629   }
2630 }
2631 
2632 void
2633 MicrosoftRecordLayoutBuilder::layoutZeroWidthBitField(const FieldDecl *FD) {
2634   // Zero-width bitfields are ignored unless they follow a non-zero-width
2635   // bitfield.
2636   if (!LastFieldIsNonZeroWidthBitfield) {
2637     placeFieldAtOffset(IsUnion ? CharUnits::Zero() : Size);
2638     // TODO: Add a Sema warning that MS ignores alignment for zero
2639     // sized bitfields that occur after zero-size bitfields or non-bitfields.
2640     return;
2641   }
2642   LastFieldIsNonZeroWidthBitfield = false;
2643   ElementInfo Info = getAdjustedElementInfo(FD);
2644   if (IsUnion) {
2645     placeFieldAtOffset(CharUnits::Zero());
2646     Size = std::max(Size, Info.Size);
2647     // TODO: Add a Sema warning that MS ignores bitfield alignment in unions.
2648   } else {
2649     // Round up the current record size to the field's alignment boundary.
2650     CharUnits FieldOffset = Size.alignTo(Info.Alignment);
2651     placeFieldAtOffset(FieldOffset);
2652     Size = FieldOffset;
2653     Alignment = std::max(Alignment, Info.Alignment);
2654   }
2655 }
2656 
2657 void MicrosoftRecordLayoutBuilder::injectVBPtr(const CXXRecordDecl *RD) {
2658   if (!HasVBPtr || SharedVBPtrBase)
2659     return;
2660   // Inject the VBPointer at the injection site.
2661   CharUnits InjectionSite = VBPtrOffset;
2662   // But before we do, make sure it's properly aligned.
2663   VBPtrOffset = VBPtrOffset.alignTo(PointerInfo.Alignment);
2664   // Shift everything after the vbptr down, unless we're using an external
2665   // layout.
2666   if (UseExternalLayout)
2667     return;
2668   // Determine where the first field should be laid out after the vbptr.
2669   CharUnits FieldStart = VBPtrOffset + PointerInfo.Size;
2670   // Make sure that the amount we push the fields back by is a multiple of the
2671   // alignment.
2672   CharUnits Offset = (FieldStart - InjectionSite)
2673                          .alignTo(std::max(RequiredAlignment, Alignment));
2674   Size += Offset;
2675   for (uint64_t &FieldOffset : FieldOffsets)
2676     FieldOffset += Context.toBits(Offset);
2677   for (BaseOffsetsMapTy::value_type &Base : Bases)
2678     if (Base.second >= InjectionSite)
2679       Base.second += Offset;
2680 }
2681 
2682 void MicrosoftRecordLayoutBuilder::injectVFPtr(const CXXRecordDecl *RD) {
2683   if (!HasOwnVFPtr)
2684     return;
2685   // Make sure that the amount we push the struct back by is a multiple of the
2686   // alignment.
2687   CharUnits Offset =
2688       PointerInfo.Size.alignTo(std::max(RequiredAlignment, Alignment));
2689   // Push back the vbptr, but increase the size of the object and push back
2690   // regular fields by the offset only if not using external record layout.
2691   if (HasVBPtr)
2692     VBPtrOffset += Offset;
2693 
2694   if (UseExternalLayout)
2695     return;
2696 
2697   Size += Offset;
2698 
2699   // If we're using an external layout, the fields offsets have already
2700   // accounted for this adjustment.
2701   for (uint64_t &FieldOffset : FieldOffsets)
2702     FieldOffset += Context.toBits(Offset);
2703   for (BaseOffsetsMapTy::value_type &Base : Bases)
2704     Base.second += Offset;
2705 }
2706 
2707 void MicrosoftRecordLayoutBuilder::layoutVirtualBases(const CXXRecordDecl *RD) {
2708   if (!HasVBPtr)
2709     return;
2710   // Vtordisps are always 4 bytes (even in 64-bit mode)
2711   CharUnits VtorDispSize = CharUnits::fromQuantity(4);
2712   CharUnits VtorDispAlignment = VtorDispSize;
2713   // vtordisps respect pragma pack.
2714   if (!MaxFieldAlignment.isZero())
2715     VtorDispAlignment = std::min(VtorDispAlignment, MaxFieldAlignment);
2716   // The alignment of the vtordisp is at least the required alignment of the
2717   // entire record.  This requirement may be present to support vtordisp
2718   // injection.
2719   for (const CXXBaseSpecifier &VBase : RD->vbases()) {
2720     const CXXRecordDecl *BaseDecl = VBase.getType()->getAsCXXRecordDecl();
2721     const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(BaseDecl);
2722     RequiredAlignment =
2723         std::max(RequiredAlignment, BaseLayout.getRequiredAlignment());
2724   }
2725   VtorDispAlignment = std::max(VtorDispAlignment, RequiredAlignment);
2726   // Compute the vtordisp set.
2727   llvm::SmallPtrSet<const CXXRecordDecl *, 2> HasVtorDispSet;
2728   computeVtorDispSet(HasVtorDispSet, RD);
2729   // Iterate through the virtual bases and lay them out.
2730   const ASTRecordLayout *PreviousBaseLayout = nullptr;
2731   for (const CXXBaseSpecifier &VBase : RD->vbases()) {
2732     const CXXRecordDecl *BaseDecl = VBase.getType()->getAsCXXRecordDecl();
2733     const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(BaseDecl);
2734     bool HasVtordisp = HasVtorDispSet.count(BaseDecl) > 0;
2735     // Insert padding between two bases if the left first one is zero sized or
2736     // contains a zero sized subobject and the right is zero sized or one leads
2737     // with a zero sized base.  The padding between virtual bases is 4
2738     // bytes (in both 32 and 64 bits modes) and always involves rounding up to
2739     // the required alignment, we don't know why.
2740     if ((PreviousBaseLayout && PreviousBaseLayout->hasZeroSizedSubObject() &&
2741         BaseLayout.leadsWithZeroSizedBase()) || HasVtordisp) {
2742       Size = Size.alignTo(VtorDispAlignment) + VtorDispSize;
2743       Alignment = std::max(VtorDispAlignment, Alignment);
2744     }
2745     // Insert the virtual base.
2746     ElementInfo Info = getAdjustedElementInfo(BaseLayout);
2747     CharUnits BaseOffset;
2748 
2749     // Respect the external AST source base offset, if present.
2750     bool FoundBase = false;
2751     if (UseExternalLayout) {
2752       FoundBase = External.getExternalVBaseOffset(BaseDecl, BaseOffset);
2753       if (FoundBase)
2754         assert(BaseOffset >= Size && "base offset already allocated");
2755     }
2756     if (!FoundBase)
2757       BaseOffset = Size.alignTo(Info.Alignment);
2758 
2759     VBases.insert(std::make_pair(BaseDecl,
2760         ASTRecordLayout::VBaseInfo(BaseOffset, HasVtordisp)));
2761     Size = BaseOffset + BaseLayout.getNonVirtualSize();
2762     PreviousBaseLayout = &BaseLayout;
2763   }
2764 }
2765 
2766 void MicrosoftRecordLayoutBuilder::finalizeLayout(const RecordDecl *RD) {
2767   // Respect required alignment.  Note that in 32-bit mode Required alignment
2768   // may be 0 and cause size not to be updated.
2769   DataSize = Size;
2770   if (!RequiredAlignment.isZero()) {
2771     Alignment = std::max(Alignment, RequiredAlignment);
2772     auto RoundingAlignment = Alignment;
2773     if (!MaxFieldAlignment.isZero())
2774       RoundingAlignment = std::min(RoundingAlignment, MaxFieldAlignment);
2775     RoundingAlignment = std::max(RoundingAlignment, RequiredAlignment);
2776     Size = Size.alignTo(RoundingAlignment);
2777   }
2778   if (Size.isZero()) {
2779     EndsWithZeroSizedObject = true;
2780     LeadsWithZeroSizedBase = true;
2781     // Zero-sized structures have size equal to their alignment if a
2782     // __declspec(align) came into play.
2783     if (RequiredAlignment >= MinEmptyStructSize)
2784       Size = Alignment;
2785     else
2786       Size = MinEmptyStructSize;
2787   }
2788 
2789   if (UseExternalLayout) {
2790     Size = Context.toCharUnitsFromBits(External.Size);
2791     if (External.Align)
2792       Alignment = Context.toCharUnitsFromBits(External.Align);
2793   }
2794 }
2795 
2796 // Recursively walks the non-virtual bases of a class and determines if any of
2797 // them are in the bases with overridden methods set.
2798 static bool
2799 RequiresVtordisp(const llvm::SmallPtrSetImpl<const CXXRecordDecl *> &
2800                      BasesWithOverriddenMethods,
2801                  const CXXRecordDecl *RD) {
2802   if (BasesWithOverriddenMethods.count(RD))
2803     return true;
2804   // If any of a virtual bases non-virtual bases (recursively) requires a
2805   // vtordisp than so does this virtual base.
2806   for (const CXXBaseSpecifier &Base : RD->bases())
2807     if (!Base.isVirtual() &&
2808         RequiresVtordisp(BasesWithOverriddenMethods,
2809                          Base.getType()->getAsCXXRecordDecl()))
2810       return true;
2811   return false;
2812 }
2813 
2814 void MicrosoftRecordLayoutBuilder::computeVtorDispSet(
2815     llvm::SmallPtrSetImpl<const CXXRecordDecl *> &HasVtordispSet,
2816     const CXXRecordDecl *RD) const {
2817   // /vd2 or #pragma vtordisp(2): Always use vtordisps for virtual bases with
2818   // vftables.
2819   if (RD->getMSVtorDispMode() == MSVtorDispAttr::ForVFTable) {
2820     for (const CXXBaseSpecifier &Base : RD->vbases()) {
2821       const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
2822       const ASTRecordLayout &Layout = Context.getASTRecordLayout(BaseDecl);
2823       if (Layout.hasExtendableVFPtr())
2824         HasVtordispSet.insert(BaseDecl);
2825     }
2826     return;
2827   }
2828 
2829   // If any of our bases need a vtordisp for this type, so do we.  Check our
2830   // direct bases for vtordisp requirements.
2831   for (const CXXBaseSpecifier &Base : RD->bases()) {
2832     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
2833     const ASTRecordLayout &Layout = Context.getASTRecordLayout(BaseDecl);
2834     for (const auto &bi : Layout.getVBaseOffsetsMap())
2835       if (bi.second.hasVtorDisp())
2836         HasVtordispSet.insert(bi.first);
2837   }
2838   // We don't introduce any additional vtordisps if either:
2839   // * A user declared constructor or destructor aren't declared.
2840   // * #pragma vtordisp(0) or the /vd0 flag are in use.
2841   if ((!RD->hasUserDeclaredConstructor() && !RD->hasUserDeclaredDestructor()) ||
2842       RD->getMSVtorDispMode() == MSVtorDispAttr::Never)
2843     return;
2844   // /vd1 or #pragma vtordisp(1): Try to guess based on whether we think it's
2845   // possible for a partially constructed object with virtual base overrides to
2846   // escape a non-trivial constructor.
2847   assert(RD->getMSVtorDispMode() == MSVtorDispAttr::ForVBaseOverride);
2848   // Compute a set of base classes which define methods we override.  A virtual
2849   // base in this set will require a vtordisp.  A virtual base that transitively
2850   // contains one of these bases as a non-virtual base will also require a
2851   // vtordisp.
2852   llvm::SmallPtrSet<const CXXMethodDecl *, 8> Work;
2853   llvm::SmallPtrSet<const CXXRecordDecl *, 2> BasesWithOverriddenMethods;
2854   // Seed the working set with our non-destructor, non-pure virtual methods.
2855   for (const CXXMethodDecl *MD : RD->methods())
2856     if (MD->isVirtual() && !isa<CXXDestructorDecl>(MD) && !MD->isPure())
2857       Work.insert(MD);
2858   while (!Work.empty()) {
2859     const CXXMethodDecl *MD = *Work.begin();
2860     CXXMethodDecl::method_iterator i = MD->begin_overridden_methods(),
2861                                    e = MD->end_overridden_methods();
2862     // If a virtual method has no-overrides it lives in its parent's vtable.
2863     if (i == e)
2864       BasesWithOverriddenMethods.insert(MD->getParent());
2865     else
2866       Work.insert(i, e);
2867     // We've finished processing this element, remove it from the working set.
2868     Work.erase(MD);
2869   }
2870   // For each of our virtual bases, check if it is in the set of overridden
2871   // bases or if it transitively contains a non-virtual base that is.
2872   for (const CXXBaseSpecifier &Base : RD->vbases()) {
2873     const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
2874     if (!HasVtordispSet.count(BaseDecl) &&
2875         RequiresVtordisp(BasesWithOverriddenMethods, BaseDecl))
2876       HasVtordispSet.insert(BaseDecl);
2877   }
2878 }
2879 
2880 /// getASTRecordLayout - Get or compute information about the layout of the
2881 /// specified record (struct/union/class), which indicates its size and field
2882 /// position information.
2883 const ASTRecordLayout &
2884 ASTContext::getASTRecordLayout(const RecordDecl *D) const {
2885   // These asserts test different things.  A record has a definition
2886   // as soon as we begin to parse the definition.  That definition is
2887   // not a complete definition (which is what isDefinition() tests)
2888   // until we *finish* parsing the definition.
2889 
2890   if (D->hasExternalLexicalStorage() && !D->getDefinition())
2891     getExternalSource()->CompleteType(const_cast<RecordDecl*>(D));
2892 
2893   D = D->getDefinition();
2894   assert(D && "Cannot get layout of forward declarations!");
2895   assert(!D->isInvalidDecl() && "Cannot get layout of invalid decl!");
2896   assert(D->isCompleteDefinition() && "Cannot layout type before complete!");
2897 
2898   // Look up this layout, if already laid out, return what we have.
2899   // Note that we can't save a reference to the entry because this function
2900   // is recursive.
2901   const ASTRecordLayout *Entry = ASTRecordLayouts[D];
2902   if (Entry) return *Entry;
2903 
2904   const ASTRecordLayout *NewEntry = nullptr;
2905 
2906   if (isMsLayout(*this)) {
2907     MicrosoftRecordLayoutBuilder Builder(*this);
2908     if (const auto *RD = dyn_cast<CXXRecordDecl>(D)) {
2909       Builder.cxxLayout(RD);
2910       NewEntry = new (*this) ASTRecordLayout(
2911           *this, Builder.Size, Builder.Alignment, Builder.RequiredAlignment,
2912           Builder.HasOwnVFPtr, Builder.HasOwnVFPtr || Builder.PrimaryBase,
2913           Builder.VBPtrOffset, Builder.NonVirtualSize,
2914           Builder.FieldOffsets.data(), Builder.FieldOffsets.size(),
2915           Builder.NonVirtualSize, Builder.Alignment, CharUnits::Zero(),
2916           Builder.PrimaryBase, false, Builder.SharedVBPtrBase,
2917           Builder.EndsWithZeroSizedObject, Builder.LeadsWithZeroSizedBase,
2918           Builder.Bases, Builder.VBases);
2919     } else {
2920       Builder.layout(D);
2921       NewEntry = new (*this) ASTRecordLayout(
2922           *this, Builder.Size, Builder.Alignment, Builder.RequiredAlignment,
2923           Builder.Size, Builder.FieldOffsets.data(),
2924           Builder.FieldOffsets.size());
2925     }
2926   } else {
2927     if (const auto *RD = dyn_cast<CXXRecordDecl>(D)) {
2928       EmptySubobjectMap EmptySubobjects(*this, RD);
2929       ItaniumRecordLayoutBuilder Builder(*this, &EmptySubobjects);
2930       Builder.Layout(RD);
2931 
2932       // In certain situations, we are allowed to lay out objects in the
2933       // tail-padding of base classes.  This is ABI-dependent.
2934       // FIXME: this should be stored in the record layout.
2935       bool skipTailPadding =
2936           mustSkipTailPadding(getTargetInfo().getCXXABI(), RD);
2937 
2938       // FIXME: This should be done in FinalizeLayout.
2939       CharUnits DataSize =
2940           skipTailPadding ? Builder.getSize() : Builder.getDataSize();
2941       CharUnits NonVirtualSize =
2942           skipTailPadding ? DataSize : Builder.NonVirtualSize;
2943       NewEntry = new (*this) ASTRecordLayout(
2944           *this, Builder.getSize(), Builder.Alignment,
2945           /*RequiredAlignment : used by MS-ABI)*/
2946           Builder.Alignment, Builder.HasOwnVFPtr, RD->isDynamicClass(),
2947           CharUnits::fromQuantity(-1), DataSize, Builder.FieldOffsets.data(),
2948           Builder.FieldOffsets.size(), NonVirtualSize,
2949           Builder.NonVirtualAlignment,
2950           EmptySubobjects.SizeOfLargestEmptySubobject, Builder.PrimaryBase,
2951           Builder.PrimaryBaseIsVirtual, nullptr, false, false, Builder.Bases,
2952           Builder.VBases);
2953     } else {
2954       ItaniumRecordLayoutBuilder Builder(*this, /*EmptySubobjects=*/nullptr);
2955       Builder.Layout(D);
2956 
2957       NewEntry = new (*this) ASTRecordLayout(
2958           *this, Builder.getSize(), Builder.Alignment,
2959           /*RequiredAlignment : used by MS-ABI)*/
2960           Builder.Alignment, Builder.getSize(), Builder.FieldOffsets.data(),
2961           Builder.FieldOffsets.size());
2962     }
2963   }
2964 
2965   ASTRecordLayouts[D] = NewEntry;
2966 
2967   if (getLangOpts().DumpRecordLayouts) {
2968     llvm::outs() << "\n*** Dumping AST Record Layout\n";
2969     DumpRecordLayout(D, llvm::outs(), getLangOpts().DumpRecordLayoutsSimple);
2970   }
2971 
2972   return *NewEntry;
2973 }
2974 
2975 const CXXMethodDecl *ASTContext::getCurrentKeyFunction(const CXXRecordDecl *RD) {
2976   if (!getTargetInfo().getCXXABI().hasKeyFunctions())
2977     return nullptr;
2978 
2979   assert(RD->getDefinition() && "Cannot get key function for forward decl!");
2980   RD = cast<CXXRecordDecl>(RD->getDefinition());
2981 
2982   // Beware:
2983   //  1) computing the key function might trigger deserialization, which might
2984   //     invalidate iterators into KeyFunctions
2985   //  2) 'get' on the LazyDeclPtr might also trigger deserialization and
2986   //     invalidate the LazyDeclPtr within the map itself
2987   LazyDeclPtr Entry = KeyFunctions[RD];
2988   const Decl *Result =
2989       Entry ? Entry.get(getExternalSource()) : computeKeyFunction(*this, RD);
2990 
2991   // Store it back if it changed.
2992   if (Entry.isOffset() || Entry.isValid() != bool(Result))
2993     KeyFunctions[RD] = const_cast<Decl*>(Result);
2994 
2995   return cast_or_null<CXXMethodDecl>(Result);
2996 }
2997 
2998 void ASTContext::setNonKeyFunction(const CXXMethodDecl *Method) {
2999   assert(Method == Method->getFirstDecl() &&
3000          "not working with method declaration from class definition");
3001 
3002   // Look up the cache entry.  Since we're working with the first
3003   // declaration, its parent must be the class definition, which is
3004   // the correct key for the KeyFunctions hash.
3005   const auto &Map = KeyFunctions;
3006   auto I = Map.find(Method->getParent());
3007 
3008   // If it's not cached, there's nothing to do.
3009   if (I == Map.end()) return;
3010 
3011   // If it is cached, check whether it's the target method, and if so,
3012   // remove it from the cache. Note, the call to 'get' might invalidate
3013   // the iterator and the LazyDeclPtr object within the map.
3014   LazyDeclPtr Ptr = I->second;
3015   if (Ptr.get(getExternalSource()) == Method) {
3016     // FIXME: remember that we did this for module / chained PCH state?
3017     KeyFunctions.erase(Method->getParent());
3018   }
3019 }
3020 
3021 static uint64_t getFieldOffset(const ASTContext &C, const FieldDecl *FD) {
3022   const ASTRecordLayout &Layout = C.getASTRecordLayout(FD->getParent());
3023   return Layout.getFieldOffset(FD->getFieldIndex());
3024 }
3025 
3026 uint64_t ASTContext::getFieldOffset(const ValueDecl *VD) const {
3027   uint64_t OffsetInBits;
3028   if (const FieldDecl *FD = dyn_cast<FieldDecl>(VD)) {
3029     OffsetInBits = ::getFieldOffset(*this, FD);
3030   } else {
3031     const IndirectFieldDecl *IFD = cast<IndirectFieldDecl>(VD);
3032 
3033     OffsetInBits = 0;
3034     for (const NamedDecl *ND : IFD->chain())
3035       OffsetInBits += ::getFieldOffset(*this, cast<FieldDecl>(ND));
3036   }
3037 
3038   return OffsetInBits;
3039 }
3040 
3041 /// getObjCLayout - Get or compute information about the layout of the
3042 /// given interface.
3043 ///
3044 /// \param Impl - If given, also include the layout of the interface's
3045 /// implementation. This may differ by including synthesized ivars.
3046 const ASTRecordLayout &
3047 ASTContext::getObjCLayout(const ObjCInterfaceDecl *D,
3048                           const ObjCImplementationDecl *Impl) const {
3049   // Retrieve the definition
3050   if (D->hasExternalLexicalStorage() && !D->getDefinition())
3051     getExternalSource()->CompleteType(const_cast<ObjCInterfaceDecl*>(D));
3052   D = D->getDefinition();
3053   assert(D && D->isThisDeclarationADefinition() && "Invalid interface decl!");
3054 
3055   // Look up this layout, if already laid out, return what we have.
3056   const ObjCContainerDecl *Key =
3057     Impl ? (const ObjCContainerDecl*) Impl : (const ObjCContainerDecl*) D;
3058   if (const ASTRecordLayout *Entry = ObjCLayouts[Key])
3059     return *Entry;
3060 
3061   // Add in synthesized ivar count if laying out an implementation.
3062   if (Impl) {
3063     unsigned SynthCount = CountNonClassIvars(D);
3064     // If there aren't any sythesized ivars then reuse the interface
3065     // entry. Note we can't cache this because we simply free all
3066     // entries later; however we shouldn't look up implementations
3067     // frequently.
3068     if (SynthCount == 0)
3069       return getObjCLayout(D, nullptr);
3070   }
3071 
3072   ItaniumRecordLayoutBuilder Builder(*this, /*EmptySubobjects=*/nullptr);
3073   Builder.Layout(D);
3074 
3075   const ASTRecordLayout *NewEntry =
3076     new (*this) ASTRecordLayout(*this, Builder.getSize(),
3077                                 Builder.Alignment,
3078                                 /*RequiredAlignment : used by MS-ABI)*/
3079                                 Builder.Alignment,
3080                                 Builder.getDataSize(),
3081                                 Builder.FieldOffsets.data(),
3082                                 Builder.FieldOffsets.size());
3083 
3084   ObjCLayouts[Key] = NewEntry;
3085 
3086   return *NewEntry;
3087 }
3088 
3089 static void PrintOffset(raw_ostream &OS,
3090                         CharUnits Offset, unsigned IndentLevel) {
3091   OS << llvm::format("%10" PRId64 " | ", (int64_t)Offset.getQuantity());
3092   OS.indent(IndentLevel * 2);
3093 }
3094 
3095 static void PrintBitFieldOffset(raw_ostream &OS, CharUnits Offset,
3096                                 unsigned Begin, unsigned Width,
3097                                 unsigned IndentLevel) {
3098   llvm::SmallString<10> Buffer;
3099   {
3100     llvm::raw_svector_ostream BufferOS(Buffer);
3101     BufferOS << Offset.getQuantity() << ':';
3102     if (Width == 0) {
3103       BufferOS << '-';
3104     } else {
3105       BufferOS << Begin << '-' << (Begin + Width - 1);
3106     }
3107   }
3108 
3109   OS << llvm::right_justify(Buffer, 10) << " | ";
3110   OS.indent(IndentLevel * 2);
3111 }
3112 
3113 static void PrintIndentNoOffset(raw_ostream &OS, unsigned IndentLevel) {
3114   OS << "           | ";
3115   OS.indent(IndentLevel * 2);
3116 }
3117 
3118 static void DumpRecordLayout(raw_ostream &OS, const RecordDecl *RD,
3119                              const ASTContext &C,
3120                              CharUnits Offset,
3121                              unsigned IndentLevel,
3122                              const char* Description,
3123                              bool PrintSizeInfo,
3124                              bool IncludeVirtualBases) {
3125   const ASTRecordLayout &Layout = C.getASTRecordLayout(RD);
3126   auto CXXRD = dyn_cast<CXXRecordDecl>(RD);
3127 
3128   PrintOffset(OS, Offset, IndentLevel);
3129   OS << C.getTypeDeclType(const_cast<RecordDecl*>(RD)).getAsString();
3130   if (Description)
3131     OS << ' ' << Description;
3132   if (CXXRD && CXXRD->isEmpty())
3133     OS << " (empty)";
3134   OS << '\n';
3135 
3136   IndentLevel++;
3137 
3138   // Dump bases.
3139   if (CXXRD) {
3140     const CXXRecordDecl *PrimaryBase = Layout.getPrimaryBase();
3141     bool HasOwnVFPtr = Layout.hasOwnVFPtr();
3142     bool HasOwnVBPtr = Layout.hasOwnVBPtr();
3143 
3144     // Vtable pointer.
3145     if (CXXRD->isDynamicClass() && !PrimaryBase && !isMsLayout(C)) {
3146       PrintOffset(OS, Offset, IndentLevel);
3147       OS << '(' << *RD << " vtable pointer)\n";
3148     } else if (HasOwnVFPtr) {
3149       PrintOffset(OS, Offset, IndentLevel);
3150       // vfptr (for Microsoft C++ ABI)
3151       OS << '(' << *RD << " vftable pointer)\n";
3152     }
3153 
3154     // Collect nvbases.
3155     SmallVector<const CXXRecordDecl *, 4> Bases;
3156     for (const CXXBaseSpecifier &Base : CXXRD->bases()) {
3157       assert(!Base.getType()->isDependentType() &&
3158              "Cannot layout class with dependent bases.");
3159       if (!Base.isVirtual())
3160         Bases.push_back(Base.getType()->getAsCXXRecordDecl());
3161     }
3162 
3163     // Sort nvbases by offset.
3164     std::stable_sort(Bases.begin(), Bases.end(),
3165                      [&](const CXXRecordDecl *L, const CXXRecordDecl *R) {
3166       return Layout.getBaseClassOffset(L) < Layout.getBaseClassOffset(R);
3167     });
3168 
3169     // Dump (non-virtual) bases
3170     for (const CXXRecordDecl *Base : Bases) {
3171       CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base);
3172       DumpRecordLayout(OS, Base, C, BaseOffset, IndentLevel,
3173                        Base == PrimaryBase ? "(primary base)" : "(base)",
3174                        /*PrintSizeInfo=*/false,
3175                        /*IncludeVirtualBases=*/false);
3176     }
3177 
3178     // vbptr (for Microsoft C++ ABI)
3179     if (HasOwnVBPtr) {
3180       PrintOffset(OS, Offset + Layout.getVBPtrOffset(), IndentLevel);
3181       OS << '(' << *RD << " vbtable pointer)\n";
3182     }
3183   }
3184 
3185   // Dump fields.
3186   uint64_t FieldNo = 0;
3187   for (RecordDecl::field_iterator I = RD->field_begin(),
3188          E = RD->field_end(); I != E; ++I, ++FieldNo) {
3189     const FieldDecl &Field = **I;
3190     uint64_t LocalFieldOffsetInBits = Layout.getFieldOffset(FieldNo);
3191     CharUnits FieldOffset =
3192       Offset + C.toCharUnitsFromBits(LocalFieldOffsetInBits);
3193 
3194     // Recursively dump fields of record type.
3195     if (auto RT = Field.getType()->getAs<RecordType>()) {
3196       DumpRecordLayout(OS, RT->getDecl(), C, FieldOffset, IndentLevel,
3197                        Field.getName().data(),
3198                        /*PrintSizeInfo=*/false,
3199                        /*IncludeVirtualBases=*/true);
3200       continue;
3201     }
3202 
3203     if (Field.isBitField()) {
3204       uint64_t LocalFieldByteOffsetInBits = C.toBits(FieldOffset - Offset);
3205       unsigned Begin = LocalFieldOffsetInBits - LocalFieldByteOffsetInBits;
3206       unsigned Width = Field.getBitWidthValue(C);
3207       PrintBitFieldOffset(OS, FieldOffset, Begin, Width, IndentLevel);
3208     } else {
3209       PrintOffset(OS, FieldOffset, IndentLevel);
3210     }
3211     OS << Field.getType().getAsString() << ' ' << Field << '\n';
3212   }
3213 
3214   // Dump virtual bases.
3215   if (CXXRD && IncludeVirtualBases) {
3216     const ASTRecordLayout::VBaseOffsetsMapTy &VtorDisps =
3217       Layout.getVBaseOffsetsMap();
3218 
3219     for (const CXXBaseSpecifier &Base : CXXRD->vbases()) {
3220       assert(Base.isVirtual() && "Found non-virtual class!");
3221       const CXXRecordDecl *VBase = Base.getType()->getAsCXXRecordDecl();
3222 
3223       CharUnits VBaseOffset = Offset + Layout.getVBaseClassOffset(VBase);
3224 
3225       if (VtorDisps.find(VBase)->second.hasVtorDisp()) {
3226         PrintOffset(OS, VBaseOffset - CharUnits::fromQuantity(4), IndentLevel);
3227         OS << "(vtordisp for vbase " << *VBase << ")\n";
3228       }
3229 
3230       DumpRecordLayout(OS, VBase, C, VBaseOffset, IndentLevel,
3231                        VBase == Layout.getPrimaryBase() ?
3232                          "(primary virtual base)" : "(virtual base)",
3233                        /*PrintSizeInfo=*/false,
3234                        /*IncludeVirtualBases=*/false);
3235     }
3236   }
3237 
3238   if (!PrintSizeInfo) return;
3239 
3240   PrintIndentNoOffset(OS, IndentLevel - 1);
3241   OS << "[sizeof=" << Layout.getSize().getQuantity();
3242   if (CXXRD && !isMsLayout(C))
3243     OS << ", dsize=" << Layout.getDataSize().getQuantity();
3244   OS << ", align=" << Layout.getAlignment().getQuantity();
3245 
3246   if (CXXRD) {
3247     OS << ",\n";
3248     PrintIndentNoOffset(OS, IndentLevel - 1);
3249     OS << " nvsize=" << Layout.getNonVirtualSize().getQuantity();
3250     OS << ", nvalign=" << Layout.getNonVirtualAlignment().getQuantity();
3251   }
3252   OS << "]\n";
3253 }
3254 
3255 void ASTContext::DumpRecordLayout(const RecordDecl *RD,
3256                                   raw_ostream &OS,
3257                                   bool Simple) const {
3258   if (!Simple) {
3259     ::DumpRecordLayout(OS, RD, *this, CharUnits(), 0, nullptr,
3260                        /*PrintSizeInfo*/true,
3261                        /*IncludeVirtualBases=*/true);
3262     return;
3263   }
3264 
3265   // The "simple" format is designed to be parsed by the
3266   // layout-override testing code.  There shouldn't be any external
3267   // uses of this format --- when LLDB overrides a layout, it sets up
3268   // the data structures directly --- so feel free to adjust this as
3269   // you like as long as you also update the rudimentary parser for it
3270   // in libFrontend.
3271 
3272   const ASTRecordLayout &Info = getASTRecordLayout(RD);
3273   OS << "Type: " << getTypeDeclType(RD).getAsString() << "\n";
3274   OS << "\nLayout: ";
3275   OS << "<ASTRecordLayout\n";
3276   OS << "  Size:" << toBits(Info.getSize()) << "\n";
3277   if (!isMsLayout(*this))
3278     OS << "  DataSize:" << toBits(Info.getDataSize()) << "\n";
3279   OS << "  Alignment:" << toBits(Info.getAlignment()) << "\n";
3280   OS << "  FieldOffsets: [";
3281   for (unsigned i = 0, e = Info.getFieldCount(); i != e; ++i) {
3282     if (i) OS << ", ";
3283     OS << Info.getFieldOffset(i);
3284   }
3285   OS << "]>\n";
3286 }
3287